@@ -1,5 +1,5 @@
 /**
- * Copyright © 2017-2025 Wellington Wallace
+ * Copyright © 2017-2026 Wellington Wallace
  *
  * This file is part of Easy Effects.
  *
@@ -70,7 +70,6 @@ Autogain::~Autogain() {
     disconnect_from_pw();
   }
 
-  disconnect();
   settings->disconnect();
 
   if (ebur_state != nullptr) {
@@ -84,13 +83,15 @@ void Autogain::reset() {
   settings->setDefaults();
 }
 
+void Autogain::clear_data() {
+  setup();
+}
+
 auto Autogain::init_ebur128() -> bool {
   if (n_samples == 0U || rate == 0U) {
     return false;
   }
 
-  internal_output_gain = 1.0;
-
   if (ebur_state != nullptr) {
     ebur128_destroy(&ebur_state);
 
@@ -118,8 +119,29 @@ void Autogain::set_maximum_history(const int& seconds) {
 }
 
 void Autogain::setup() {
+  if (rate == 0 || n_samples == 0) {
+    // Some signals may be emitted before PipeWire calls our setup function
+    return;
+  }
+
   std::scoped_lock<std::mutex> lock(data_mutex);
 
+  block_time = static_cast<double>(n_samples) / static_cast<double>(rate);
+
+  attack_coeff = std::exp(-block_time / attack_time);
+  release_coeff = std::exp(-block_time / release_time);
+
+  if (2U * static_cast<size_t>(n_samples) != data.size()) {
+    data.resize(static_cast<size_t>(n_samples) * 2U);
+  }
+
+  // There is no need to reset libebur128 when n_samples change.
+  // Only rate changes matter for it.
+
+  if (ebur128_ready && rate == old_rate) {
+    return;
+  }
+
   ebur128_ready = false;
 
   // NOLINTBEGIN(clang-analyzer-cplusplus.NewDeleteLeaks)
@@ -130,15 +152,6 @@ void Autogain::setup() {
           return;
         }
 
-        if (2U * static_cast<size_t>(n_samples) != data.size()) {
-          data.resize(static_cast<size_t>(n_samples) * 2U);
-        }
-
-        block_time = static_cast<double>(n_samples) / static_cast<double>(rate);
-
-        attack_coeff = std::exp(-block_time / attack_time);
-        release_coeff = std::exp(-block_time / release_time);
-
         auto status = true;
 
         old_rate = rate;
@@ -159,7 +172,7 @@ void Autogain::process(std::span<float>& left_in,
                        std::span<float>& right_out) {
   std::scoped_lock<std::mutex> lock(data_mutex);
 
-  if (bypass || !ebur128_ready) {
+  if (bypass) {
     std::ranges::copy(left_in, left_out.begin());
     std::ranges::copy(right_in, right_out.begin());
 
@@ -170,6 +183,19 @@ void Autogain::process(std::span<float>& left_in,
     apply_gain(left_in, right_in, input_gain);
   }
 
+  if (!ebur128_ready) {
+    std::ranges::copy(left_in, left_out.begin());
+    std::ranges::copy(right_in, right_out.begin());
+
+    const float final_gain = static_cast<float>(internal_output_gain) * output_gain;
+
+    if (final_gain != 1.0F) {
+      apply_gain(left_out, right_out, final_gain);
+    }
+
+    return;
+  }
+
   {
     const float* __restrict__ l = left_in.data();
     const float* __restrict__ r = right_in.data();
@@ -365,5 +391,13 @@ float Autogain::getOutputGainLevel() const {
 }
 
 void Autogain::resetHistory() {
+  internal_output_gain = 1.0;
+
+  data_mutex.lock();
+
+  ebur128_ready = false;
+
+  data_mutex.unlock();
+
   setup();
 }
