@@ -241,7 +241,16 @@ void Convolver::reset() {
   settings->setDefaults();
 }
 
+void Convolver::clear_data() {
+  setup();
+}
+
 void Convolver::setup() {
+  if (rate == 0 || n_samples == 0) {
+    // Some signals may be emitted before PipeWire calls our setup function
+    return;
+  }
+
   std::scoped_lock<std::mutex> lock(data_mutex);
 
   ready = false;
