@@ -1,5 +1,5 @@
 /**
- * Copyright © 2017-2025 Wellington Wallace
+ * Copyright © 2017-2026 Wellington Wallace
  *
  * This file is part of Easy Effects.
  *
@@ -38,6 +38,7 @@
 #include "pipeline_type.hpp"
 #include "plugin_base.hpp"
 #include "pw_manager.hpp"
+#include "resampler.hpp"
 #include "tags_plugin_name.hpp"
 #include "util.hpp"
 
@@ -90,12 +91,6 @@ Crystalizer::Crystalizer(const std::string& tag, pw::Manager* pipe_manager, Pipe
     frequencies[n] = f_edges[n];
   }
 
-  for (uint n = 0; n < nbands; n++) {
-    freq_scaling[n] = std::sqrt(freq_ref / freq_centers[n]);
-    // freq_scaling[n] = std::cbrt(freq_ref / freq_centers[n]);
-    // freq_scaling[n] = freq_ref / freq_centers[n];
-  }
-
   init_common_controls<db::Crystalizer>(settings);
 
   BIND_BAND(0);
@@ -111,22 +106,45 @@ Crystalizer::Crystalizer(const std::string& tag, pw::Manager* pipe_manager, Pipe
   BIND_BAND(10);
   BIND_BAND(11);
   BIND_BAND(12);
+
+  connect(settings, &db::Crystalizer::useFixedQuantumChanged, [&]() { setup(); });
+
+  connect(settings, &db::Crystalizer::oversamplingChanged, [&]() { setup(); });
+
+  connect(settings, &db::Crystalizer::transitionBandChanged, [&]() { setup(); });
+
+  connect(settings, &db::Crystalizer::oversamplingQualityChanged, [&]() {
+    std::scoped_lock<std::mutex> lock(data_mutex);
+
+    if (resampler_inL) {
+      resampler_inL->set_quality(settings->oversamplingQuality());
+    }
+
+    if (resampler_inR) {
+      resampler_inR->set_quality(settings->oversamplingQuality());
+    }
+
+    if (resampler_outL) {
+      resampler_outL->set_quality(settings->oversamplingQuality());
+    }
+
+    if (resampler_outR) {
+      resampler_outR->set_quality(settings->oversamplingQuality());
+    }
+  });
 }
 
 Crystalizer::~Crystalizer() {
+  std::scoped_lock<std::mutex> lock(data_mutex);
+
   if (connected_to_pw) {
     disconnect_from_pw();
   }
 
-  disconnect();
   settings->disconnect();
 
-  data_mutex.lock();
-
   filters_are_ready = false;
 
-  data_mutex.unlock();
-
   util::debug(std::format("{}{} destroyed", log_tag, name.toStdString()));
 }
 
@@ -134,18 +152,26 @@ void Crystalizer::reset() {
   settings->setDefaults();
 }
 
+void Crystalizer::clear_data() {
+  setup();
+}
+
 void Crystalizer::setup() {
-  data_mutex.lock();
+  if (rate == 0 || n_samples == 0) {
+    // Some signals may be emitted before PipeWire calls our setup function
+    return;
+  }
+
+  std::scoped_lock<std::mutex> lock(data_mutex);
+
+  auto same_blocksize = settings->useFixedQuantum() ? n_samples == default_quantum : n_samples == blocksize;
+
+  if (filters_are_ready && rate == current_rate && same_blocksize) {
+    return;
+  }
 
   filters_are_ready = false;
 
-  data_mutex.unlock();
-
-  block_time = static_cast<float>(n_samples) / static_cast<float>(rate);
-
-  attack_coeff = std::exp(-block_time / attack_time);
-  release_coeff = std::exp(-block_time / release_time);
-
   /**
    * As zita uses fftw we have to be careful when reinitializing it.
    * The thread that creates the fftw plan has to be the same that destroys it.
@@ -156,15 +182,28 @@ void Crystalizer::setup() {
   // NOLINTBEGIN(clang-analyzer-cplusplus.NewDeleteLeaks)
 
   QMetaObject::invokeMethod(
-      this,
+      baseWorker,
       [this] {
         if (filters_are_ready) {
           return;
         }
 
-        blocksize = n_samples;
+        current_rate = rate;
 
-        n_samples_is_power_of_2 = (n_samples & (n_samples - 1U)) == 0 && n_samples != 0U;
+        do_oversampling = settings->oversampling();
+
+        block_time = static_cast<float>(n_samples) / static_cast<float>(rate);
+
+        attack_coeff = std::exp(-block_time / attack_time);
+        release_coeff = std::exp(-block_time / release_time);
+
+        auto blockrate = do_oversampling ? 2 * rate : rate;
+
+        blocksize = do_oversampling ? 2 * n_samples : n_samples;
+
+        blocksize = settings->useFixedQuantum() ? default_quantum : blocksize;
+
+        n_samples_is_power_of_2 = (blocksize & (blocksize - 1U)) == 0 && blocksize != 0U;
 
         if (!n_samples_is_power_of_2) {
           while ((blocksize & (blocksize - 1U)) != 0 && blocksize > 2U) {
@@ -172,6 +211,9 @@ void Crystalizer::setup() {
           }
         }
 
+        blocksize = std::max<uint>(blocksize, 64);    // zita does not work with less than 64
+        blocksize = std::min<uint>(blocksize, 8192);  // zita does not work with more that 8192
+
         util::debug(std::format("{}{} blocksize: {}", log_tag, name.toStdString(), blocksize));
 
         notify_latency = true;
@@ -215,19 +257,30 @@ void Crystalizer::setup() {
 
         for (uint n = 0U; n < nbands; n++) {
           filters.at(n)->set_n_samples(blocksize);
-          filters.at(n)->set_rate(rate);
-
+          filters.at(n)->set_rate(blockrate);
           filters.at(n)->set_min_frequency(frequencies.at(n));
           filters.at(n)->set_max_frequency(frequencies.at(n + 1U));
-
+          filters.at(n)->set_transition_band(settings->transitionBand());
           filters.at(n)->setup();
         }
 
-        data_mutex.lock();
+        resampler_inL = std::make_unique<Resampler>(rate, 2 * rate);
+        resampler_inR = std::make_unique<Resampler>(rate, 2 * rate);
+
+        resampler_outL = std::make_unique<Resampler>(2 * rate, rate);
+        resampler_outR = std::make_unique<Resampler>(2 * rate, rate);
+
+        resampler_inL->set_quality(settings->oversamplingQuality());
+
+        resampler_inR->set_quality(settings->oversamplingQuality());
+
+        resampler_outL->set_quality(settings->oversamplingQuality());
+
+        resampler_outR->set_quality(settings->oversamplingQuality());
+
+        std::scoped_lock<std::mutex> lock(data_mutex);
 
         filters_are_ready = true;
-
-        data_mutex.unlock();
       },
       Qt::QueuedConnection);
 
@@ -240,7 +293,7 @@ void Crystalizer::process(std::span<float>& left_in,
                           std::span<float>& right_out) {
   std::scoped_lock<std::mutex> lock(data_mutex);
 
-  if (bypass || !filters_are_ready) {
+  if (bypass) {
     std::ranges::copy(left_in, left_out.begin());
     std::ranges::copy(right_in, right_out.begin());
 
@@ -251,14 +304,35 @@ void Crystalizer::process(std::span<float>& left_in,
     apply_gain(left_in, right_in, input_gain);
   }
 
-  if (n_samples_is_power_of_2 && blocksize == n_samples) {
+  if (!filters_are_ready) {
+    std::ranges::copy(left_in, left_out.begin());
+    std::ranges::copy(right_in, right_out.begin());
+
+    if (output_gain != 1.0F) {
+      apply_gain(left_out, right_out, output_gain);
+    }
+
+    return;
+  }
+
+  if (n_samples_is_power_of_2 && blocksize == n_samples && !do_oversampling) {
     std::ranges::copy(left_in, left_out.begin());
     std::ranges::copy(right_in, right_out.begin());
 
     enhance_peaks(left_out, right_out);
   } else {
-    buf_in_L.insert(buf_in_L.end(), left_in.begin(), left_in.end());
-    buf_in_R.insert(buf_in_R.end(), right_in.begin(), right_in.end());
+    if (!do_oversampling) {
+      buf_in_L.insert(buf_in_L.end(), left_in.begin(), left_in.end());
+      buf_in_R.insert(buf_in_R.end(), right_in.begin(), right_in.end());
+    } else {
+      const auto resampled_inL = resampler_inL->process(left_in);
+      const auto resampled_inR = resampler_inR->process(right_in);
+
+      buf_in_L.insert(buf_in_L.end(), resampled_inL.begin(), resampled_inL.end());
+      buf_in_R.insert(buf_in_R.end(), resampled_inR.begin(), resampled_inR.end());
+    }
+
+    // util::warning(std::format("size 1: {}, size 2: {}, size 3: {}", buf_in_L.size(), left_in.size(), data_L.size()));
 
     while (buf_in_L.size() >= blocksize) {
       util::copy_bulk(buf_in_L, data_L);
@@ -266,8 +340,16 @@ void Crystalizer::process(std::span<float>& left_in,
 
       enhance_peaks(data_L, data_R);
 
-      buf_out_L.insert(buf_out_L.end(), data_L.begin(), data_L.end());
-      buf_out_R.insert(buf_out_R.end(), data_R.begin(), data_R.end());
+      if (!do_oversampling) {
+        buf_out_L.insert(buf_out_L.end(), data_L.begin(), data_L.end());
+        buf_out_R.insert(buf_out_R.end(), data_R.begin(), data_R.end());
+      } else {
+        auto resampled_outL = resampler_outL->process(data_L);
+        auto resampled_outR = resampler_outR->process(data_R);
+
+        buf_out_L.insert(buf_out_L.end(), resampled_outL.begin(), resampled_outL.end());
+        buf_out_R.insert(buf_out_R.end(), resampled_outR.begin(), resampled_outR.end());
+      }
     }
 
     // copying the processed samples to the output buffers
@@ -434,7 +516,7 @@ auto Crystalizer::compute_adaptive_intensity(const uint& band_index,
 
   env_kurtosis = (alpha * env_kurtosis) + ((1.0F - alpha) * kurtosis);
 
-  float kurtosis_ratio = isLeft ? env_kurtosis / global_kurtosis_L : env_kurtosis / global_kurtosis_R;
+  float kurtosis_ratio = isLeft ? global_kurtosis_L / env_kurtosis : global_kurtosis_R / env_kurtosis;
 
   // crest calculation
 
@@ -463,7 +545,7 @@ auto Crystalizer::compute_adaptive_intensity(const uint& band_index,
 
   // intensity calculation
 
-  auto intensity = base_intensity * std::cbrtf(crest_ratio * kurtosis_ratio * flux_ratio) * freq_scaling[band_index];
+  auto intensity = base_intensity * std::cbrtf(crest_ratio * kurtosis_ratio * flux_ratio);
 
   // util::warning(std::format("n = {}, intensity = {}, kurtosis_r = {}, crest_r = {}, flux_r = {}", band_index,
   // intensity,
@@ -533,4 +615,4 @@ float Crystalizer::getBandFrequency(const int& index) {
 
 QList<float> Crystalizer::getAdaptiveIntensities() {
   return adaptive_intensities;
-}
\ No newline at end of file
+}
