@@ -1,5 +1,5 @@
 /**
- * Copyright © 2023 Torge Matthies
+ * Copyright © 2023-2026 Torge Matthies
  *
  * This file is part of Easy Effects.
  *
@@ -82,7 +82,6 @@ DeepFilterNet::~DeepFilterNet() {
     disconnect_from_pw();
   }
 
-  disconnect();
   settings->disconnect();
 
   util::debug(std::format("{}{} destroyed", log_tag, name.toStdString()));
@@ -92,9 +91,26 @@ void DeepFilterNet::reset() {
   settings->setDefaults();
 }
 
+void DeepFilterNet::clear_data() {
+  if (ladspa_wrapper->has_instance()) {
+    ladspa_wrapper->deactivate();
+
+    ladspa_wrapper = std::make_unique<ladspa::LadspaWrapper>("libdeep_filter_ladspa.so", "deep_filter_stereo");
+  }
+
+  setup();
+}
+
 void DeepFilterNet::setup() {
+  if (rate == 0 || n_samples == 0) {
+    // Some signals may be emitted before PipeWire calls our setup function
+    return;
+  }
+
   std::scoped_lock<std::mutex> lock(data_mutex);
 
+  ready = false;
+
   if (!ladspa_wrapper->found_plugin()) {
     return;
   }
@@ -105,10 +121,8 @@ void DeepFilterNet::setup() {
   // NOLINTBEGIN(clang-analyzer-cplusplus.NewDeleteLeaks)
 
   QMetaObject::invokeMethod(
-      this,
+      baseWorker,
       [this] {
-        std::scoped_lock<std::mutex> lock(data_mutex);
-
         if (ladspa_wrapper->has_instance()) {
           ladspa_wrapper->deactivate();
 
@@ -117,7 +131,6 @@ void DeepFilterNet::setup() {
 
         ladspa_wrapper->n_samples = n_samples;
         ladspa_wrapper->create_instance(48000);
-        ladspa_wrapper->activate();
 
         if (resample && !resampler_ready) {
           resampler_inL = std::make_unique<Resampler>(rate, 48000);
@@ -127,14 +140,14 @@ void DeepFilterNet::setup() {
 
           std::vector<float> dummy(n_samples);
 
-          const auto resampled_inL = resampler_inL->process(dummy, false);
-          const auto resampled_inR = resampler_inR->process(dummy, false);
+          const auto resampled_inL = resampler_inL->process(dummy);
+          const auto resampled_inR = resampler_inR->process(dummy);
 
           resampled_outL.resize(resampled_inL.size());
           resampled_outR.resize(resampled_inR.size());
 
-          resampler_outL->process(resampled_inL, false);
-          resampler_outR->process(resampled_inR, false);
+          resampler_outL->process(resampled_inL);
+          resampler_outR->process(resampled_inR);
 
           carryover_l.clear();
           carryover_r.clear();
@@ -145,6 +158,10 @@ void DeepFilterNet::setup() {
 
           resampler_ready = true;
         }
+
+        std::scoped_lock<std::mutex> lock(data_mutex);
+
+        ready = true;
       },
       Qt::QueuedConnection);
 
@@ -157,7 +174,7 @@ void DeepFilterNet::process(std::span<float>& left_in,
                             std::span<float>& right_out) {
   std::scoped_lock<std::mutex> lock(data_mutex);
 
-  if (!ladspa_wrapper->found_plugin() || !ladspa_wrapper->has_instance() || bypass) {
+  if (!ready || bypass) {
     std::ranges::copy(left_in, left_out.begin());
     std::ranges::copy(right_in, right_out.begin());
 
@@ -169,8 +186,8 @@ void DeepFilterNet::process(std::span<float>& left_in,
   }
 
   if (resample) {
-    const auto& resampled_inL = resampler_inL->process(left_in, false);
-    const auto& resampled_inR = resampler_inR->process(right_in, false);
+    const auto& resampled_inL = resampler_inL->process(left_in);
+    const auto& resampled_inR = resampler_inR->process(right_in);
 
     resampled_outL.resize(resampled_inL.size());
     resampled_outR.resize(resampled_inR.size());
@@ -184,19 +201,19 @@ void DeepFilterNet::process(std::span<float>& left_in,
   ladspa_wrapper->run();
 
   if (resample) {
-    const auto& outL = resampler_outL->process(resampled_outL, false);
-    const auto& outR = resampler_outR->process(resampled_outR, false);
+    const auto& outL = resampler_outL->process(resampled_outL);
+    const auto& outR = resampler_outR->process(resampled_outR);
 
-    auto carryover_end_l = std::min(carryover_l.size(), left_out.size());
-    auto carryover_end_r = std::min(carryover_r.size(), right_out.size());
+    const auto carryover_end_l = std::min(carryover_l.size(), left_out.size());
+    const auto carryover_end_r = std::min(carryover_r.size(), right_out.size());
 
-    auto left_offset =
+    const auto left_offset =
         carryover_end_l + outL.size() > left_out.size() ? carryover_end_l : left_out.size() - outL.size();
-    auto right_offset =
+    const auto right_offset =
         carryover_end_r + outR.size() > right_out.size() ? carryover_end_r : right_out.size() - outR.size();
 
-    auto left_count = std::min(outL.size(), left_out.size() - left_offset);
-    auto right_count = std::min(outR.size(), right_out.size() - right_offset);
+    const auto left_count = std::min(outL.size(), left_out.size() - left_offset);
+    const auto right_count = std::min(outR.size(), right_out.size() - right_offset);
 
     std::copy(carryover_l.begin(), carryover_l.begin() + carryover_end_l, left_out.begin());
     std::copy(carryover_r.begin(), carryover_r.begin() + carryover_end_r, right_out.begin());
