@@ -1,5 +1,5 @@
 /**
- * Copyright © 2017-2025 Wellington Wallace
+ * Copyright © 2017-2026 Wellington Wallace
  *
  * This file is part of Easy Effects.
  *
@@ -18,10 +18,13 @@
  */
 
 #include "deesser.hpp"
+#include <qnamespace.h>
 #include <qobject.h>
+#include <QApplication>
 #include <algorithm>
 #include <format>
 #include <memory>
+#include <mutex>
 #include <span>
 #include <string>
 #include "db_manager.hpp"
@@ -75,7 +78,6 @@ Deesser::~Deesser() {
     disconnect_from_pw();
   }
 
-  disconnect();
   settings->disconnect();
 
   util::debug(std::format("{}{} destroyed", log_tag, name.toStdString()));
@@ -84,29 +86,74 @@ void Deesser::reset() {
   settings->setDefaults();
 }
 
+void Deesser::clear_data() {
+  if (lv2_wrapper == nullptr) {
+    return;
+  }
+
+  {
+    std::scoped_lock<std::mutex> lock(data_mutex);
+
+    lv2_wrapper->destroy_instance();
+  }
+
+  setup();
+}
+
 void Deesser::setup() {
+  if (rate == 0 || n_samples == 0) {
+    // Some signals may be emitted before PipeWire calls our setup function
+    return;
+  }
+
+  std::scoped_lock<std::mutex> lock(data_mutex);
+
   if (!lv2_wrapper->found_plugin) {
     return;
   }
 
+  ready = false;
+
   lv2_wrapper->set_n_samples(n_samples);
 
-  if (lv2_wrapper->get_rate() != rate) {
-    lv2_wrapper->create_instance(rate);
-  }
+  // NOLINTBEGIN(clang-analyzer-cplusplus.NewDeleteLeaks)
+  QMetaObject::invokeMethod(
+      baseWorker,
+      [this] {
+        lv2_wrapper->create_instance(rate);
+
+        std::scoped_lock<std::mutex> lock(data_mutex);
+
+        ready = true;
+      },
+      Qt::QueuedConnection);
+  // NOLINTEND(clang-analyzer-cplusplus.NewDeleteLeaks)
 }
 
 void Deesser::process(std::span<float>& left_in,
                       std::span<float>& right_in,
                       std::span<float>& left_out,
                       std::span<float>& right_out) {
-  if (!lv2_wrapper->found_plugin || !lv2_wrapper->has_instance() || bypass) {
+  std::scoped_lock<std::mutex> lock(data_mutex);
+
+  if (bypass) {
     std::ranges::copy(left_in, left_out.begin());
     std::ranges::copy(right_in, right_out.begin());
 
     return;
   }
 
+  if (!ready) {
+    std::ranges::copy(left_in, left_out.begin());
+    std::ranges::copy(right_in, right_out.begin());
+
+    if (output_gain != 1.0F) {
+      apply_gain(left_out, right_out, output_gain);
+    }
+
+    return;
+  }
+
   if (input_gain != 1.0F) {
     apply_gain(left_in, right_in, input_gain);
   }
