@@ -1,5 +1,5 @@
 /**
- * Copyright © 2017-2025 Wellington Wallace
+ * Copyright © 2017-2026 Wellington Wallace
  *
  * This file is part of Easy Effects.
  *
@@ -24,6 +24,7 @@
 #include <qnamespace.h>
 #include <qobjectdefs.h>
 #include <qpoint.h>
+#include <qthread.h>
 #include <qtmetamacros.h>
 #include <qtypes.h>
 #include <spa/utils/defs.h>
@@ -43,6 +44,7 @@
 #include "compressor.hpp"
 #include "convolver.hpp"
 #include "crossfeed.hpp"
+#include "crusher.hpp"
 #include "crystalizer.hpp"
 #include "db_manager.hpp"
 #include "deepfilternet.hpp"
@@ -72,9 +74,13 @@
 #include "stereo_tools.hpp"
 #include "tags_plugin_name.hpp"
 #include "util.hpp"
+#include "voice_suppressor.hpp"
 
 EffectsBase::EffectsBase(pw::Manager* pipe_manager, PipelineType pipe_type)
-    : log_tag(pipe_type == PipelineType::output ? "soe: " : "sie: "), pm(pipe_manager), pipeline_type(pipe_type) {
+    : log_tag(pipe_type == PipelineType::output ? "soe: " : "sie: "),
+      pm(pipe_manager),
+      pipeline_type(pipe_type),
+      baseWorker(new EffectsBaseWorker) {
   using namespace std::string_literals;
 
   output_level = std::make_shared<OutputLevel>(log_tag, pm, pipeline_type, "0");
@@ -93,28 +99,39 @@ EffectsBase::EffectsBase(pw::Manager* pipe_manager, PipelineType pipe_type)
 
   switch (pipeline_type) {
     case PipelineType::input:
-      connect(db::StreamInputs::self(), &db::StreamInputs::pluginsChanged, [&]() { create_filters_if_necessary(); });
+      connect(DbStreamInputs::self(), &DbStreamInputs::pluginsChanged, [&]() { create_filters_if_necessary(); });
       break;
     case PipelineType::output:
-      connect(db::StreamOutputs::self(), &db::StreamOutputs::pluginsChanged, [&]() { create_filters_if_necessary(); });
+      connect(DbStreamOutputs::self(), &DbStreamOutputs::pluginsChanged, [&]() { create_filters_if_necessary(); });
       break;
   }
 
-  connect(db::Main::self(), &db::Main::lv2uiUpdateFrequencyChanged, [&]() {
-    auto v = db::Main::lv2uiUpdateFrequency();
+  connect(DbMain::self(), &DbMain::lv2uiUpdateFrequencyChanged, [&]() {
+    auto v = DbMain::lv2uiUpdateFrequency();
 
     for (auto& plugin : plugins | std::views::values) {
       plugin->set_native_ui_update_frequency(v);
     }
   });
+
+  // worker thread for the native ui and maybe also other things
+
+  baseWorker->moveToThread(&workerThread);
+
+  workerThread.start();
+
+  connect(&workerThread, &QThread::finished, baseWorker, &QObject::deleteLater);
 }
 
 EffectsBase::~EffectsBase() {
+  workerThread.quit();
+  workerThread.wait();
+
   util::debug("effects_base: destroyed");
 }
 
 void EffectsBase::create_filters_if_necessary() {
-  auto list = (pipeline_type == PipelineType::output ? db::StreamOutputs::plugins() : db::StreamInputs::plugins());
+  auto list = (pipeline_type == PipelineType::output ? DbStreamOutputs::plugins() : DbStreamInputs::plugins());
 
   if (list.empty()) {
     return;
@@ -127,62 +144,94 @@ void EffectsBase::create_filters_if_necessary() {
 
     auto instance_id = tags::plugin_name::get_id(name);
 
-    std::shared_ptr<PluginBase> filter = nullptr;
+    std::unique_ptr<PluginBase> filter = nullptr;
 
     if (name.startsWith(tags::plugin_name::BaseName::autogain)) {
-      filter = std::make_shared<Autogain>(log_tag, pm, pipeline_type, instance_id);
+      filter = std::make_unique<Autogain>(log_tag, pm, pipeline_type, instance_id);
+
     } else if (name.startsWith(tags::plugin_name::BaseName::bassEnhancer)) {
-      filter = std::make_shared<BassEnhancer>(log_tag, pm, pipeline_type, instance_id);
+      filter = std::make_unique<BassEnhancer>(log_tag, pm, pipeline_type, instance_id);
+
     } else if (name.startsWith(tags::plugin_name::BaseName::bassLoudness)) {
-      filter = std::make_shared<BassLoudness>(log_tag, pm, pipeline_type, instance_id);
+      filter = std::make_unique<BassLoudness>(log_tag, pm, pipeline_type, instance_id);
+
     } else if (name.startsWith(tags::plugin_name::BaseName::compressor)) {
-      filter = std::make_shared<Compressor>(log_tag, pm, pipeline_type, instance_id);
+      filter = std::make_unique<Compressor>(log_tag, pm, pipeline_type, instance_id);
+
     } else if (name.startsWith(tags::plugin_name::BaseName::convolver)) {
-      filter = std::make_shared<Convolver>(log_tag, pm, pipeline_type, instance_id);
+      filter = std::make_unique<Convolver>(log_tag, pm, pipeline_type, instance_id);
+
     } else if (name.startsWith(tags::plugin_name::BaseName::crossfeed)) {
-      filter = std::make_shared<Crossfeed>(log_tag, pm, pipeline_type, instance_id);
+      filter = std::make_unique<Crossfeed>(log_tag, pm, pipeline_type, instance_id);
+
+    } else if (name.startsWith(tags::plugin_name::BaseName::crusher)) {
+      filter = std::make_unique<Crusher>(log_tag, pm, pipeline_type, instance_id);
+
     } else if (name.startsWith(tags::plugin_name::BaseName::crystalizer)) {
-      filter = std::make_shared<Crystalizer>(log_tag, pm, pipeline_type, instance_id);
+      filter = std::make_unique<Crystalizer>(log_tag, pm, pipeline_type, instance_id);
+
     } else if (name.startsWith(tags::plugin_name::BaseName::deepfilternet)) {
-      filter = std::make_shared<DeepFilterNet>(log_tag, pm, pipeline_type, instance_id);
+      filter = std::make_unique<DeepFilterNet>(log_tag, pm, pipeline_type, instance_id);
+
     } else if (name.startsWith(tags::plugin_name::BaseName::deesser)) {
-      filter = std::make_shared<Deesser>(log_tag, pm, pipeline_type, instance_id);
+      filter = std::make_unique<Deesser>(log_tag, pm, pipeline_type, instance_id);
+
     } else if (name.startsWith(tags::plugin_name::BaseName::delay)) {
-      filter = std::make_shared<Delay>(log_tag, pm, pipeline_type, instance_id);
+      filter = std::make_unique<Delay>(log_tag, pm, pipeline_type, instance_id);
+
     } else if (name.startsWith(tags::plugin_name::BaseName::echoCanceller)) {
-      filter = std::make_shared<EchoCanceller>(log_tag, pm, pipeline_type, instance_id);
+      filter = std::make_unique<EchoCanceller>(log_tag, pm, pipeline_type, instance_id);
+
     } else if (name.startsWith(tags::plugin_name::BaseName::exciter)) {
-      filter = std::make_shared<Exciter>(log_tag, pm, pipeline_type, instance_id);
+      filter = std::make_unique<Exciter>(log_tag, pm, pipeline_type, instance_id);
+
     } else if (name.startsWith(tags::plugin_name::BaseName::expander)) {
-      filter = std::make_shared<Expander>(log_tag, pm, pipeline_type, instance_id);
+      filter = std::make_unique<Expander>(log_tag, pm, pipeline_type, instance_id);
+
     } else if (name.startsWith(tags::plugin_name::BaseName::equalizer)) {
-      filter = std::make_shared<Equalizer>(log_tag, pm, pipeline_type, instance_id);
+      filter = std::make_unique<Equalizer>(log_tag, pm, pipeline_type, instance_id);
+
     } else if (name.startsWith(tags::plugin_name::BaseName::filter)) {
-      filter = std::make_shared<Filter>(log_tag, pm, pipeline_type, instance_id);
+      filter = std::make_unique<Filter>(log_tag, pm, pipeline_type, instance_id);
+
     } else if (name.startsWith(tags::plugin_name::BaseName::gate)) {
-      filter = std::make_shared<Gate>(log_tag, pm, pipeline_type, instance_id);
+      filter = std::make_unique<Gate>(log_tag, pm, pipeline_type, instance_id);
+
+    } else if (name.startsWith(tags::plugin_name::BaseName::voiceSuppressor)) {
+      filter = std::make_unique<VoiceSuppressor>(log_tag, pm, pipeline_type, instance_id);
+
     } else if (name.startsWith(tags::plugin_name::BaseName::levelMeter)) {
-      filter = std::make_shared<LevelMeter>(log_tag, pm, pipeline_type, instance_id);
+      filter = std::make_unique<LevelMeter>(log_tag, pm, pipeline_type, instance_id);
+
     } else if (name.startsWith(tags::plugin_name::BaseName::limiter)) {
-      filter = std::make_shared<Limiter>(log_tag, pm, pipeline_type, instance_id);
+      filter = std::make_unique<Limiter>(log_tag, pm, pipeline_type, instance_id);
+
     } else if (name.startsWith(tags::plugin_name::BaseName::loudness)) {
-      filter = std::make_shared<Loudness>(log_tag, pm, pipeline_type, instance_id);
+      filter = std::make_unique<Loudness>(log_tag, pm, pipeline_type, instance_id);
+
     } else if (name.startsWith(tags::plugin_name::BaseName::maximizer)) {
-      filter = std::make_shared<Maximizer>(log_tag, pm, pipeline_type, instance_id);
+      filter = std::make_unique<Maximizer>(log_tag, pm, pipeline_type, instance_id);
+
     } else if (name.startsWith(tags::plugin_name::BaseName::multibandCompressor)) {
-      filter = std::make_shared<MultibandCompressor>(log_tag, pm, pipeline_type, instance_id);
+      filter = std::make_unique<MultibandCompressor>(log_tag, pm, pipeline_type, instance_id);
+
     } else if (name.startsWith(tags::plugin_name::BaseName::multibandGate)) {
-      filter = std::make_shared<MultibandGate>(log_tag, pm, pipeline_type, instance_id);
+      filter = std::make_unique<MultibandGate>(log_tag, pm, pipeline_type, instance_id);
+
     } else if (name.startsWith(tags::plugin_name::BaseName::pitch)) {
-      filter = std::make_shared<Pitch>(log_tag, pm, pipeline_type, instance_id);
+      filter = std::make_unique<Pitch>(log_tag, pm, pipeline_type, instance_id);
+
     } else if (name.startsWith(tags::plugin_name::BaseName::reverb)) {
-      filter = std::make_shared<Reverb>(log_tag, pm, pipeline_type, instance_id);
+      filter = std::make_unique<Reverb>(log_tag, pm, pipeline_type, instance_id);
+
     } else if (name.startsWith(tags::plugin_name::BaseName::rnnoise)) {
-      filter = std::make_shared<RNNoise>(log_tag, pm, pipeline_type, instance_id);
+      filter = std::make_unique<RNNoise>(log_tag, pm, pipeline_type, instance_id);
+
     } else if (name.startsWith(tags::plugin_name::BaseName::speex)) {
-      filter = std::make_shared<Speex>(log_tag, pm, pipeline_type, instance_id);
+      filter = std::make_unique<Speex>(log_tag, pm, pipeline_type, instance_id);
+
     } else if (name.startsWith(tags::plugin_name::BaseName::stereoTools)) {
-      filter = std::make_shared<StereoTools>(log_tag, pm, pipeline_type, instance_id);
+      filter = std::make_unique<StereoTools>(log_tag, pm, pipeline_type, instance_id);
     }
 
     if (filter != nullptr) {
@@ -196,12 +245,12 @@ void EffectsBase::create_filters_if_necessary() {
       filter->setParent(this);
     }
 
-    plugins.insert(std::make_pair(name, filter));
+    plugins.insert(std::make_pair(name, std::move(filter)));
   }
 }
 
 void EffectsBase::remove_unused_filters() {
-  auto list = (pipeline_type == PipelineType::output ? db::StreamOutputs::plugins() : db::StreamInputs::plugins());
+  auto list = (pipeline_type == PipelineType::output ? DbStreamOutputs::plugins() : DbStreamInputs::plugins());
 
   if (list.empty()) {
     plugins.clear();
@@ -213,7 +262,7 @@ void EffectsBase::remove_unused_filters() {
     auto key = it->first;
 
     if (std::ranges::find(list, key) == list.end()) {
-      auto plugin = it->second;
+      auto plugin = std::move(it->second);  // Move the unique_ptr
 
       if (plugin == nullptr) {
         it = plugins.erase(it);
@@ -246,7 +295,7 @@ void EffectsBase::deactivate_filters() {
   }
 }
 
-auto EffectsBase::get_plugins_map() -> std::map<QString, std::shared_ptr<PluginBase>> {
+auto EffectsBase::get_plugins_map() -> std::map<QString, std::unique_ptr<PluginBase>>& {
   return plugins;
 }
 
@@ -255,60 +304,92 @@ QVariant EffectsBase::getPluginInstance(const QString& pluginName) {
     return {};
   }
 
-  auto p = plugins[pluginName];
+  auto& p = plugins[pluginName];
 
   if (pluginName.startsWith(tags::plugin_name::BaseName::autogain)) {
     return QVariant::fromValue(dynamic_cast<Autogain*>(p.get()));
+
   } else if (pluginName.startsWith(tags::plugin_name::BaseName::bassEnhancer)) {
     return QVariant::fromValue(dynamic_cast<BassEnhancer*>(p.get()));
+
   } else if (pluginName.startsWith(tags::plugin_name::BaseName::bassLoudness)) {
     return QVariant::fromValue(dynamic_cast<BassLoudness*>(p.get()));
+
   } else if (pluginName.startsWith(tags::plugin_name::BaseName::compressor)) {
     return QVariant::fromValue(dynamic_cast<Compressor*>(p.get()));
+
   } else if (pluginName.startsWith(tags::plugin_name::BaseName::convolver)) {
     return QVariant::fromValue(dynamic_cast<Convolver*>(p.get()));
+
   } else if (pluginName.startsWith(tags::plugin_name::BaseName::crossfeed)) {
     return QVariant::fromValue(dynamic_cast<Crossfeed*>(p.get()));
+
+  } else if (pluginName.startsWith(tags::plugin_name::BaseName::crusher)) {
+    return QVariant::fromValue(dynamic_cast<Crusher*>(p.get()));
+
   } else if (pluginName.startsWith(tags::plugin_name::BaseName::crystalizer)) {
     return QVariant::fromValue(dynamic_cast<Crystalizer*>(p.get()));
+
   } else if (pluginName.startsWith(tags::plugin_name::BaseName::delay)) {
     return QVariant::fromValue(dynamic_cast<Delay*>(p.get()));
+
   } else if (pluginName.startsWith(tags::plugin_name::BaseName::deepfilternet)) {
     return QVariant::fromValue(dynamic_cast<DeepFilterNet*>(p.get()));
+
   } else if (pluginName.startsWith(tags::plugin_name::BaseName::deesser)) {
     return QVariant::fromValue(dynamic_cast<Deesser*>(p.get()));
+
   } else if (pluginName.startsWith(tags::plugin_name::BaseName::echoCanceller)) {
     return QVariant::fromValue(dynamic_cast<EchoCanceller*>(p.get()));
+
   } else if (pluginName.startsWith(tags::plugin_name::BaseName::equalizer)) {
     return QVariant::fromValue(dynamic_cast<Equalizer*>(p.get()));
+
   } else if (pluginName.startsWith(tags::plugin_name::BaseName::exciter)) {
     return QVariant::fromValue(dynamic_cast<Exciter*>(p.get()));
+
   } else if (pluginName.startsWith(tags::plugin_name::BaseName::expander)) {
     return QVariant::fromValue(dynamic_cast<Expander*>(p.get()));
+
   } else if (pluginName.startsWith(tags::plugin_name::BaseName::filter)) {
     return QVariant::fromValue(dynamic_cast<Filter*>(p.get()));
+
   } else if (pluginName.startsWith(tags::plugin_name::BaseName::gate)) {
     return QVariant::fromValue(dynamic_cast<Gate*>(p.get()));
+
+  } else if (pluginName.startsWith(tags::plugin_name::BaseName::voiceSuppressor)) {
+    return QVariant::fromValue(dynamic_cast<VoiceSuppressor*>(p.get()));
+
   } else if (pluginName.startsWith(tags::plugin_name::BaseName::levelMeter)) {
     return QVariant::fromValue(dynamic_cast<LevelMeter*>(p.get()));
+
   } else if (pluginName.startsWith(tags::plugin_name::BaseName::limiter)) {
     return QVariant::fromValue(dynamic_cast<Limiter*>(p.get()));
+
   } else if (pluginName.startsWith(tags::plugin_name::BaseName::loudness)) {
     return QVariant::fromValue(dynamic_cast<Loudness*>(p.get()));
+
   } else if (pluginName.startsWith(tags::plugin_name::BaseName::maximizer)) {
     return QVariant::fromValue(dynamic_cast<Maximizer*>(p.get()));
+
   } else if (pluginName.startsWith(tags::plugin_name::BaseName::multibandCompressor)) {
     return QVariant::fromValue(dynamic_cast<MultibandCompressor*>(p.get()));
+
   } else if (pluginName.startsWith(tags::plugin_name::BaseName::multibandGate)) {
     return QVariant::fromValue(dynamic_cast<MultibandGate*>(p.get()));
+
   } else if (pluginName.startsWith(tags::plugin_name::BaseName::pitch)) {
     return QVariant::fromValue(dynamic_cast<Pitch*>(p.get()));
+
   } else if (pluginName.startsWith(tags::plugin_name::BaseName::reverb)) {
     return QVariant::fromValue(dynamic_cast<Reverb*>(p.get()));
+
   } else if (pluginName.startsWith(tags::plugin_name::BaseName::rnnoise)) {
     return QVariant::fromValue(dynamic_cast<RNNoise*>(p.get()));
+
   } else if (pluginName.startsWith(tags::plugin_name::BaseName::speex)) {
     return QVariant::fromValue(dynamic_cast<Speex*>(p.get()));
+
   } else if (pluginName.startsWith(tags::plugin_name::BaseName::stereoTools)) {
     return QVariant::fromValue(dynamic_cast<StereoTools*>(p.get()));
   }
@@ -319,13 +400,12 @@ QVariant EffectsBase::getPluginInstance(const QString& pluginName) {
 uint EffectsBase::getPipeLineRate() const {
   switch (pipeline_type) {
     case PipelineType::input:
-      if (auto node = pm->model_nodes.get_node_by_name(db::StreamInputs::inputDevice());
-          node.serial != SPA_ID_INVALID) {
+      if (auto node = pm->model_nodes.get_node_by_name(DbStreamInputs::inputDevice()); node.serial != SPA_ID_INVALID) {
         return node.rate * 0.001F;
       }
       return 0.0F;
     case PipelineType::output: {
-      if (auto node = pm->model_nodes.get_node_by_name(db::StreamOutputs::outputDevice());
+      if (auto node = pm->model_nodes.get_node_by_name(DbStreamOutputs::outputDevice());
           node.serial != SPA_ID_INVALID) {
         return node.rate * 0.001F;
       }
@@ -337,7 +417,7 @@ uint EffectsBase::getPipeLineRate() const {
 }
 
 uint EffectsBase::getPipeLineLatency() {
-  auto list = (pipeline_type == PipelineType::output ? db::StreamOutputs::plugins() : db::StreamInputs::plugins());
+  auto list = (pipeline_type == PipelineType::output ? DbStreamOutputs::plugins() : DbStreamInputs::plugins());
 
   auto v = 0.0F;
 
@@ -369,7 +449,7 @@ void EffectsBase::requestSpectrumData() {
   // NOLINTBEGIN(clang-analyzer-cplusplus.NewDeleteLeaks)
 
   QMetaObject::invokeMethod(
-      this,
+      baseWorker,
       [this] {
         auto [rate, list] = spectrum->compute_magnitudes();
 
@@ -385,8 +465,8 @@ void EffectsBase::requestSpectrumData() {
           frequencies[n] = 0.5F * static_cast<float>(rate) * static_cast<float>(n) / static_cast<float>(n_bands);
         }
 
-        const auto min_freq = static_cast<float>(db::Spectrum::minimumFrequency());
-        const auto max_freq = static_cast<float>(db::Spectrum::maximumFrequency());
+        const auto min_freq = static_cast<float>(DbSpectrum::minimumFrequency());
+        const auto max_freq = static_cast<float>(DbSpectrum::maximumFrequency());
 
         if (min_freq > (max_freq - 100.0F)) {
           return;
@@ -394,10 +474,10 @@ void EffectsBase::requestSpectrumData() {
 
         std::vector<float> x_axis;
 
-        if (db::Spectrum::logarithimicHorizontalAxis()) {
-          x_axis = util::logspace(min_freq, max_freq, db::Spectrum::nPoints());
+        if (DbSpectrum::logarithimicHorizontalAxis()) {
+          x_axis = util::logspace(min_freq, max_freq, DbSpectrum::nPoints());
         } else {
-          x_axis = util::linspace(min_freq, max_freq, db::Spectrum::nPoints());
+          x_axis = util::linspace(min_freq, max_freq, DbSpectrum::nPoints());
         }
 
         auto* acc = gsl_interp_accel_alloc();
@@ -443,4 +523,4 @@ void EffectsBase::setUpdateLevelMeters(const bool& state) {
 
 void EffectsBase::setSpectrumBypass(const bool& state) {
   spectrum->bypass = state;
-}
\ No newline at end of file
+}
