@@ -19,14 +19,17 @@
 
 #include "equalizer.hpp"
 #include <qlist.h>
+#include <qnamespace.h>
 #include <qobject.h>
 #include <sys/types.h>
+#include <QApplication>
 #include <algorithm>
 #include <cmath>
 #include <filesystem>
 #include <format>
 #include <map>
 #include <memory>
+#include <mutex>
 #include <span>
 #include <string>
 #include <utility>
@@ -185,29 +188,74 @@ void Equalizer::on_split_channels() {
   UNIFIED_BANDS_PROPERTY_BIND(settings_right, settings_left, Gain);
 }
 
+void Equalizer::clear_data() {
+  if (lv2_wrapper == nullptr) {
+    return;
+  }
+
+  {
+    std::scoped_lock<std::mutex> lock(data_mutex);
+
+    lv2_wrapper->destroy_instance();
+  }
+
+  setup();
+}
+
 void Equalizer::setup() {
+  if (rate == 0 || n_samples == 0) {
+    // Some signals may be emitted before PipeWire calls our setup function
+    return;
+  }
+
+  std::scoped_lock<std::mutex> lock(data_mutex);
+
   if (!lv2_wrapper->found_plugin) {
     return;
   }
 
+  ready = false;
+
   lv2_wrapper->set_n_samples(n_samples);
 
-  if (lv2_wrapper->get_rate() != rate) {
-    lv2_wrapper->create_instance(rate);
-  }
+  // NOLINTBEGIN(clang-analyzer-cplusplus.NewDeleteLeaks)
+  QMetaObject::invokeMethod(
+      baseWorker,
+      [this] {
+        lv2_wrapper->create_instance(rate);
+
+        std::scoped_lock<std::mutex> lock(data_mutex);
+
+        ready = true;
+      },
+      Qt::QueuedConnection);
+  // NOLINTEND(clang-analyzer-cplusplus.NewDeleteLeaks)
 }
 
 void Equalizer::process(std::span<float>& left_in,
                         std::span<float>& right_in,
                         std::span<float>& left_out,
                         std::span<float>& right_out) {
-  if (!lv2_wrapper->found_plugin || !lv2_wrapper->has_instance() || bypass) {
+  std::scoped_lock<std::mutex> lock(data_mutex);
+
+  if (bypass) {
     std::ranges::copy(left_in, left_out.begin());
     std::ranges::copy(right_in, right_out.begin());
 
     return;
   }
 
+  if (!ready) {
+    std::ranges::copy(left_in, left_out.begin());
+    std::ranges::copy(right_in, right_out.begin());
+
+    if (output_gain != 1.0F) {
+      apply_gain(left_out, right_out, output_gain);
+    }
+
+    return;
+  }
+
   if (input_gain != 1.0F) {
     apply_gain(left_in, right_in, input_gain);
   }
