@@ -1,5 +1,5 @@
 /**
- * Copyright © 2017-2025 Wellington Wallace
+ * Copyright © 2017-2026 Wellington Wallace
  *
  * This file is part of Easy Effects.
  *
@@ -19,12 +19,16 @@
 
 #include "multiband_compressor.hpp"
 #include <qlist.h>
+#include <qnamespace.h>
+#include <qobjectdefs.h>
 #include <qtypes.h>
 #include <spa/utils/defs.h>
+#include <QApplication>
 #include <QString>
 #include <algorithm>
 #include <format>
 #include <memory>
+#include <mutex>
 #include <span>
 #include <string>
 #include "db_manager.hpp"
@@ -118,6 +122,20 @@ void MultibandCompressor::reset() {
   settings->setDefaults();
 }
 
+void MultibandCompressor::clear_data() {
+  if (lv2_wrapper == nullptr) {
+    return;
+  }
+
+  {
+    std::scoped_lock<std::mutex> lock(data_mutex);
+
+    lv2_wrapper->destroy_instance();
+  }
+
+  setup();
+}
+
 // NOLINTNEXTLINE(readability-function-size,hicpp-function-size)
 void MultibandCompressor::bind_bands() {
   using namespace tags::multiband_compressor;
@@ -175,15 +193,33 @@ void MultibandCompressor::bind_bands() {
 }
 
 void MultibandCompressor::setup() {
+  if (rate == 0 || n_samples == 0) {
+    // Some signals may be emitted before PipeWire calls our setup function
+    return;
+  }
+
+  std::scoped_lock<std::mutex> lock(data_mutex);
+
   if (!lv2_wrapper->found_plugin) {
     return;
   }
 
+  ready = false;
+
   lv2_wrapper->set_n_samples(n_samples);
 
-  if (lv2_wrapper->get_rate() != rate) {
-    lv2_wrapper->create_instance(rate);
-  }
+  // NOLINTBEGIN(clang-analyzer-cplusplus.NewDeleteLeaks)
+  QMetaObject::invokeMethod(
+      baseWorker,
+      [this] {
+        lv2_wrapper->create_instance(rate);
+
+        std::scoped_lock<std::mutex> lock(data_mutex);
+
+        ready = true;
+      },
+      Qt::QueuedConnection);
+  // NOLINTEND(clang-analyzer-cplusplus.NewDeleteLeaks)
 }
 
 void MultibandCompressor::process([[maybe_unused]] std::span<float>& left_in,
@@ -197,13 +233,26 @@ void MultibandCompressor::process(std::span<float>& left_in,
                                   std::span<float>& right_out,
                                   std::span<float>& probe_left,
                                   std::span<float>& probe_right) {
-  if (!lv2_wrapper->found_plugin || !lv2_wrapper->has_instance() || bypass) {
+  std::scoped_lock<std::mutex> lock(data_mutex);
+
+  if (bypass) {
     std::ranges::copy(left_in, left_out.begin());
     std::ranges::copy(right_in, right_out.begin());
 
     return;
   }
 
+  if (!ready) {
+    std::ranges::copy(left_in, left_out.begin());
+    std::ranges::copy(right_in, right_out.begin());
+
+    if (output_gain != 1.0F) {
+      apply_gain(left_out, right_out, output_gain);
+    }
+
+    return;
+  }
+
   if (input_gain != 1.0F) {
     apply_gain(left_in, right_in, input_gain);
   }
