@@ -1,5 +1,5 @@
 /**
- * Copyright © 2017-2025 Wellington Wallace
+ * Copyright © 2017-2026 Wellington Wallace
  *
  * This file is part of Easy Effects.
  *
@@ -91,7 +91,6 @@ Pitch::~Pitch() {
     disconnect_from_pw();
   }
 
-  disconnect();
   settings->disconnect();
 
   util::debug(std::format("{}{} destroyed", log_tag, name.toStdString()));
@@ -101,25 +100,36 @@ void Pitch::reset() {
   settings->setDefaults();
 }
 
+void Pitch::clear_data() {
+  setup();
+}
+
 void Pitch::setup() {
-  soundtouch_ready = false;
-
-  latency_n_frames = 0U;
-
-  if (data.size() != static_cast<size_t>(n_samples) * 2) {
-    data.resize(2U * static_cast<size_t>(n_samples));
+  if (rate == 0 || n_samples == 0) {
+    // Some signals may be emitted before PipeWire calls our setup function
+    return;
   }
 
-  deque_out_L.resize(0U);
-  deque_out_R.resize(0U);
+  std::scoped_lock<std::mutex> lock(data_mutex);
+
+  soundtouch_ready = false;
 
   QMetaObject::invokeMethod(
-      this,
+      baseWorker,
       [this] {
         if (soundtouch_ready) {
           return;
         }
 
+        latency_n_frames = 0U;
+
+        if (data.size() != static_cast<size_t>(n_samples) * 2) {
+          data.resize(2U * static_cast<size_t>(n_samples));
+        }
+
+        deque_out_L.resize(0U);
+        deque_out_R.resize(0U);
+
         init_soundtouch();
 
         std::scoped_lock<std::mutex> lock(data_mutex);
@@ -337,24 +347,7 @@ auto Pitch::get_latency_seconds() -> float {
 void Pitch::resetHistory() {
   // NOLINTBEGIN(clang-analyzer-cplusplus.NewDeleteLeaks)
 
-  QMetaObject::invokeMethod(
-      this,
-      [this] {
-        data_mutex.lock();
-
-        soundtouch_ready = false;
-
-        data_mutex.unlock();
-
-        init_soundtouch();
-
-        data_mutex.lock();
-
-        soundtouch_ready = true;
-
-        data_mutex.unlock();
-      },
-      Qt::QueuedConnection);
+  QMetaObject::invokeMethod(baseWorker, [this] { setup(); }, Qt::QueuedConnection);
 
   // NOLINTEND(clang-analyzer-cplusplus.NewDeleteLeaks)
 }
