@@ -1,5 +1,5 @@
 /**
- * Copyright © 2017-2025 Wellington Wallace
+ * Copyright © 2017-2026 Wellington Wallace
  *
  * This file is part of Easy Effects.
  *
@@ -26,6 +26,7 @@
 #include <pipewire/thread-loop.h>
 #include <qnamespace.h>
 #include <qobjectdefs.h>
+#include <qthread.h>
 #include <qtimer.h>
 #include <spa/node/io.h>
 #include <spa/param/latency-utils.h>
@@ -82,6 +83,11 @@ void on_process(void* userdata, spa_io_position* position) {
     std::ranges::fill(d->pb->dummy_right, 0.0F);
   }
 
+  if (DbMain::copyFilterInputBuffers() && d->pb->copy_left_in.size() != n_samples) {
+    d->pb->copy_left_in.resize(n_samples);
+    d->pb->copy_right_in.resize(n_samples);
+  }
+
   if (rate != d->pb->rate || n_samples != d->pb->n_samples) {
     d->pb->rate = rate;
     d->pb->n_samples = n_samples;
@@ -110,6 +116,11 @@ void on_process(void* userdata, spa_io_position* position) {
 
   if (in_left != nullptr) {
     left_in = std::span(in_left, n_samples);
+
+    if (DbMain::copyFilterInputBuffers()) {
+      std::ranges::copy(left_in, d->pb->copy_left_in.begin());
+    }
+
   } else {
     if (!d->pb->got_null_left_in) {
       util::warning("Processing: we received a null left_in pointer. Using the dummy array instead.");
@@ -122,6 +133,11 @@ void on_process(void* userdata, spa_io_position* position) {
 
   if (in_right != nullptr) {
     right_in = std::span(in_right, n_samples);
+
+    if (DbMain::copyFilterInputBuffers()) {
+      std::ranges::copy(right_in, d->pb->copy_right_in.begin());
+    }
+
   } else {
     if (!d->pb->got_null_right_in) {
       util::warning("Processing: we received a null right_in pointer. Using the dummy array instead.");
@@ -157,7 +173,15 @@ void on_process(void* userdata, spa_io_position* position) {
   }
 
   if (!d->pb->enable_probe) {
-    d->pb->process(left_in, right_in, left_out, right_out);
+    if (DbMain::copyFilterInputBuffers()) {
+      auto copy_left_in = std::span(d->pb->copy_left_in);
+      auto copy_right_in = std::span(d->pb->copy_right_in);
+
+      d->pb->process(copy_left_in, copy_right_in, left_out, right_out);
+    } else {
+      d->pb->process(left_in, right_in, left_out, right_out);
+    }
+
   } else {
     auto* probe_left = static_cast<float*>(pw_filter_get_dsp_buffer(d->probe_left, n_samples));
     auto* probe_right = static_cast<float*>(pw_filter_get_dsp_buffer(d->probe_right, n_samples));
@@ -172,12 +196,27 @@ void on_process(void* userdata, spa_io_position* position) {
       std::span l(d->pb->dummy_left.data(), n_samples);
       std::span r(d->pb->dummy_right.data(), n_samples);
 
-      d->pb->process(left_in, right_in, left_out, right_out, l, r);
+      if (DbMain::copyFilterInputBuffers()) {
+        auto copy_left_in = std::span(d->pb->copy_left_in);
+        auto copy_right_in = std::span(d->pb->copy_right_in);
+
+        d->pb->process(copy_left_in, copy_right_in, left_out, right_out, l, r);
+      } else {
+        d->pb->process(left_in, right_in, left_out, right_out, l, r);
+      }
+
     } else {
       std::span l(probe_left, n_samples);
       std::span r(probe_right, n_samples);
 
-      d->pb->process(left_in, right_in, left_out, right_out, l, r);
+      if (DbMain::copyFilterInputBuffers()) {
+        auto copy_left_in = std::span(d->pb->copy_left_in);
+        auto copy_right_in = std::span(d->pb->copy_right_in);
+
+        d->pb->process(copy_left_in, copy_right_in, left_out, right_out, l, r);
+      } else {
+        d->pb->process(left_in, right_in, left_out, right_out, l, r);
+      }
     }
   }
 }
@@ -263,6 +302,7 @@ PluginBase::PluginBase(std::string tag,
       pipeline_type(pipe_type),
       enable_probe(enable_probe),
       pm(pipe_manager),
+      baseWorker(new PluginBaseWorker),
       native_ui_timer(new QTimer(this)) {
   QString description;
   QString description_pipeline;
@@ -375,7 +415,7 @@ PluginBase::PluginBase(std::string tag,
 
   pm->sync_wait_unlock();
 
-  native_ui_timer->setInterval(static_cast<long>(1000.0 / db::Main::lv2uiUpdateFrequency()));
+  native_ui_timer->setInterval(static_cast<long>(1000.0 / DbMain::lv2uiUpdateFrequency()));
 
   connect(native_ui_timer, &QTimer::timeout, this, [&]() {
     if (lv2_wrapper == nullptr || !lv2_wrapper->has_ui()) {
@@ -385,6 +425,14 @@ PluginBase::PluginBase(std::string tag,
     lv2_wrapper->notify_ui();
     lv2_wrapper->update_ui();
   });
+
+  // worker thread for the native ui and maybe also other things
+
+  baseWorker->moveToThread(&workerThread);
+
+  workerThread.start();
+
+  connect(&workerThread, &QThread::finished, baseWorker, &QObject::deleteLater);
 }
 
 PluginBase::~PluginBase() {
@@ -397,6 +445,9 @@ PluginBase::~PluginBase() {
   pw_filter_destroy(filter);
 
   pm->sync_wait_unlock();
+
+  workerThread.quit();
+  workerThread.wait();
 }
 
 void PluginBase::reset() {}
@@ -483,9 +534,11 @@ void PluginBase::disconnect_from_pw() {
     std::this_thread::sleep_for(std::chrono::milliseconds(1));
   }
 
-  util::warning(std::format("{}{} is disconnected", log_tag, name.toStdString()));
+  util::debug(std::format("{}{} is disconnected", log_tag, name.toStdString()));
 }
 
+void PluginBase::clear_data() {}
+
 void PluginBase::setup() {}
 
 void PluginBase::process([[maybe_unused]] std::span<float>& left_in,
@@ -505,36 +558,39 @@ auto PluginBase::get_latency_seconds() -> float {
 }
 
 void PluginBase::showNativeUi() {
-  if (lv2_wrapper == nullptr) {
-    return;
-  }
+  native_ui_timer->start();
 
-  if (!lv2_wrapper->has_ui()) {
-    // using invokeMethod to force this code to run in the main thread and
-    // avoid load in the QML thread.
+  // using invokeMethod to force this code to run in the base class worker thread and
+  // avoid load in QML or main thread.
 
-    // NOLINTBEGIN(clang-analyzer-cplusplus.NewDeleteLeaks)
-    QMetaObject::invokeMethod(
-        this,
-        [this] {
+  // NOLINTBEGIN(clang-analyzer-cplusplus.NewDeleteLeaks)
+  QMetaObject::invokeMethod(
+      baseWorker,
+      [this] {
+        if (lv2_wrapper != nullptr && !lv2_wrapper->has_ui()) {
           lv2_wrapper->load_ui();
-          native_ui_timer->start();
-        },
-        Qt::QueuedConnection);
+        }
+      },
+      Qt::QueuedConnection);
 
-    // NOLINTEND(clang-analyzer-cplusplus.NewDeleteLeaks)
-  }
+  // NOLINTEND(clang-analyzer-cplusplus.NewDeleteLeaks)
 }
 
 void PluginBase::closeNativeUi() {
   native_ui_timer->stop();
 
-  if (lv2_wrapper == nullptr) {
-    return;
-  }
+  // NOLINTBEGIN(clang-analyzer-cplusplus.NewDeleteLeaks)
+  QMetaObject::invokeMethod(
+      baseWorker,
+      [this] {
+        if (lv2_wrapper != nullptr) {
+          lv2_wrapper->native_ui_to_database();
+          lv2_wrapper->close_ui();
+        }
+      },
+      Qt::QueuedConnection);
 
-  lv2_wrapper->native_ui_to_database();
-  lv2_wrapper->close_ui();
+  // NOLINTEND(clang-analyzer-cplusplus.NewDeleteLeaks)
 }
 
 void PluginBase::set_native_ui_update_frequency(const uint& value) {
