@@ -1,5 +1,5 @@
 /**
- * Copyright © 2017-2025 Wellington Wallace
+ * Copyright © 2017-2026 Wellington Wallace
  *
  * This file is part of Easy Effects.
  *
@@ -51,6 +51,7 @@
 #include "config.h"
 #include "convolver_preset.hpp"
 #include "crossfeed_preset.hpp"
+#include "crusher_preset.hpp"
 #include "crystalizer_preset.hpp"
 #include "deepfilternet_preset.hpp"
 #include "deesser_preset.hpp"
@@ -84,6 +85,7 @@
 #include "stereo_tools_preset.hpp"
 #include "tags_plugin_name.hpp"
 #include "util.hpp"
+#include "voice_suppressor_preset.hpp"
 
 namespace presets {
 
@@ -98,7 +100,6 @@ Manager::Manager() : outputListModel(new ListModel(this)), inputListModel(new Li
   prepare_last_used_preset_key(PipelineType::output);
 
   connect(&autoload_manager, &AutoloadManager::loadPresetRequested, this, &Manager::loadLocalPresetFile);
-  connect(&autoload_manager, &AutoloadManager::loadFallbackPresetRequested, this, &Manager::loadLocalPresetFile);
 }
 
 void Manager::initialize_qml_types() {
@@ -120,11 +121,11 @@ void Manager::initialize_qml_types() {
                                                       community_manager.get_output_model()->getProxy());
 
   qmlRegisterSingletonInstance<QSortFilterProxyModel>("ee.presets", VERSION_MAJOR, VERSION_MINOR,
-                                                      "SortedAutoloadingInputListModel",
+                                                      "SortedAutoloadInputListModel",
                                                       autoload_manager.get_input_model()->getProxy());
 
   qmlRegisterSingletonInstance<QSortFilterProxyModel>("ee.presets", VERSION_MAJOR, VERSION_MINOR,
-                                                      "SortedAutoloadingOutputListModel",
+                                                      "SortedAutoloadOutputListModel",
                                                       autoload_manager.get_output_model()->getProxy());
 
   qmlRegisterSingletonInstance<QSortFilterProxyModel>("ee.presets", VERSION_MAJOR, VERSION_MINOR,
@@ -153,7 +154,7 @@ void Manager::prepare_filesystem_watchers() {
 
 void Manager::prepare_last_used_preset_key(const PipelineType& pipeline_type) {
   const auto preset_name =
-      pipeline_type == PipelineType::input ? db::Main::lastLoadedInputPreset() : db::Main::lastLoadedOutputPreset();
+      pipeline_type == PipelineType::input ? DbMain::lastLoadedInputPreset() : DbMain::lastLoadedOutputPreset();
 
   bool reset_key = true;
 
@@ -172,10 +173,10 @@ void Manager::prepare_last_used_preset_key(const PipelineType& pipeline_type) {
   if (reset_key) {
     switch (pipeline_type) {
       case PipelineType::input:
-        db::Main::setLastLoadedInputPreset(db::Main::defaultLastLoadedInputPresetValue());
+        DbMain::setLastLoadedInputPreset(DbMain::defaultLastLoadedInputPresetValue());
         break;
       case PipelineType::output:
-        db::Main::setLastLoadedOutputPreset(db::Main::defaultLastLoadedOutputPresetValue());
+        DbMain::setLastLoadedOutputPreset(DbMain::defaultLastLoadedOutputPresetValue());
         break;
     }
   }
@@ -194,7 +195,7 @@ void Manager::save_blocklist(const PipelineType& pipeline_type, nlohmann::json&
 
   switch (pipeline_type) {
     case PipelineType::output: {
-      const auto list = db::StreamOutputs::blocklist();
+      const auto list = DbStreamOutputs::blocklist();
 
       for (const auto& l : list) {
         blocklist.push_back(l.toStdString());
@@ -205,7 +206,7 @@ void Manager::save_blocklist(const PipelineType& pipeline_type, nlohmann::json&
       break;
     }
     case PipelineType::input: {
-      const auto list = db::StreamInputs::blocklist();
+      const auto list = DbStreamInputs::blocklist();
 
       for (const auto& l : list) {
         blocklist.push_back(l.toStdString());
@@ -219,6 +220,10 @@ void Manager::save_blocklist(const PipelineType& pipeline_type, nlohmann::json&
 }
 
 auto Manager::load_blocklist(const PipelineType& pipeline_type, const nlohmann::json& json) -> bool {
+  if (DbMain::ignorePresetBlocklist()) {
+    return true;
+  }
+
   std::vector<std::string> blocklist;
 
   switch (pipeline_type) {
@@ -232,9 +237,9 @@ auto Manager::load_blocklist(const PipelineType& pipeline_type, const nlohmann::
           new_list.append(QString::fromStdString(app));
         }
 
-        db::StreamInputs::setBlocklist(new_list);
+        DbStreamInputs::setBlocklist(new_list);
       } catch (const nlohmann::json::exception& e) {
-        db::StreamInputs::setBlocklist(QStringList{});
+        DbStreamInputs::setBlocklist(QStringList{});
 
         notify_error(PresetError::blocklist_format);
 
@@ -242,7 +247,7 @@ auto Manager::load_blocklist(const PipelineType& pipeline_type, const nlohmann::
 
         return false;
       } catch (...) {
-        db::StreamInputs::setBlocklist(QStringList{});
+        DbStreamInputs::setBlocklist(QStringList{});
 
         notify_error(PresetError::blocklist_generic);
 
@@ -261,9 +266,9 @@ auto Manager::load_blocklist(const PipelineType& pipeline_type, const nlohmann::
           new_list.append(QString::fromStdString(app));
         }
 
-        db::StreamOutputs::setBlocklist(new_list);
+        DbStreamOutputs::setBlocklist(new_list);
       } catch (const nlohmann::json::exception& e) {
-        db::StreamOutputs::setBlocklist(QStringList{});
+        DbStreamOutputs::setBlocklist(QStringList{});
 
         notify_error(PresetError::blocklist_format);
 
@@ -271,7 +276,7 @@ auto Manager::load_blocklist(const PipelineType& pipeline_type, const nlohmann::
 
         return false;
       } catch (...) {
-        db::StreamOutputs::setBlocklist(QStringList{});
+        DbStreamOutputs::setBlocklist(QStringList{});
 
         notify_error(PresetError::blocklist_generic);
 
@@ -299,7 +304,7 @@ bool Manager::savePresetFile(const PipelineType& pipeline_type, const QString& n
 
   switch (pipeline_type) {
     case PipelineType::output: {
-      const auto plugins = db::StreamOutputs::plugins();
+      const auto plugins = DbStreamOutputs::plugins();
 
       std::vector<std::string> list;
 
@@ -319,7 +324,7 @@ bool Manager::savePresetFile(const PipelineType& pipeline_type, const QString& n
       break;
     }
     case PipelineType::input: {
-      const auto plugins = db::StreamInputs::plugins();
+      const auto plugins = DbStreamInputs::plugins();
 
       std::vector<std::string> list;
 
@@ -361,6 +366,9 @@ bool Manager::add(const PipelineType& pipeline_type, const QString& name) {
 
   savePresetFile(pipeline_type, name);
 
+  // adding this here just to increase how frequently we update the most used presets list
+  update_used_presets_list(pipeline_type, "");
+
   return true;
 }
 
@@ -372,17 +380,28 @@ bool Manager::remove(const PipelineType& pipeline_type, const QString& name) {
 
   preset_file = conf_dir / std::filesystem::path{name.toStdString() + DirectoryManager::json_ext};
 
-  if (std::filesystem::exists(preset_file)) {
-    std::filesystem::remove(preset_file);
-
-    util::debug(std::format("Removed preset: {}", preset_file.string()));
-
-    update_used_presets_list(pipeline_type, "");
-
-    return true;
+  if (!std::filesystem::exists(preset_file)) {
+    return false;
   }
 
-  return false;
+  std::filesystem::remove(preset_file);
+
+  // Check if the removed name is the last local preset loaded.
+  if (pipeline_type == PipelineType::output) {
+    if (DbMain::lastLoadedOutputCommunityPackage().isEmpty() && name == DbMain::lastLoadedOutputPreset()) {
+      DbMain::setLastLoadedOutputPreset(DbMain::defaultLastLoadedOutputPresetValue());
+    }
+  } else if (pipeline_type == PipelineType::input) {
+    if (DbMain::lastLoadedInputCommunityPackage().isEmpty() && name == DbMain::lastLoadedInputPreset()) {
+      DbMain::setLastLoadedInputPreset(DbMain::defaultLastLoadedInputPresetValue());
+    }
+  }
+
+  util::debug(std::format("Removed preset: {}", preset_file.string()));
+
+  update_used_presets_list(pipeline_type, "");
+
+  return true;
 }
 
 bool Manager::renameLocalPresetFile(const PipelineType& pipeline_type, const QString& name, const QString& newName) {
@@ -398,15 +417,17 @@ bool Manager::renameLocalPresetFile(const PipelineType& pipeline_type, const QSt
 
   new_file = conf_dir / std::filesystem::path{newName.toStdString() + DirectoryManager::json_ext};
 
-  if (std::filesystem::exists(preset_file)) {
-    std::filesystem::rename(preset_file, new_file);
-
-    util::debug(std::format("Renamed preset: {} to {}", preset_file.string(), new_file.string()));
-
-    return true;
+  if (!std::filesystem::exists(preset_file)) {
+    return false;
   }
 
-  return false;
+  std::filesystem::rename(preset_file, new_file);
+
+  util::debug(std::format("Renamed preset: {} to {}", preset_file.string(), new_file.string()));
+
+  update_used_presets_list(pipeline_type, "");
+
+  return true;
 }
 
 auto Manager::read_effects_pipeline_from_preset(const PipelineType& pipeline_type,
@@ -456,10 +477,10 @@ auto Manager::read_effects_pipeline_from_preset(const PipelineType& pipeline_typ
 
   switch (pipeline_type) {
     case PipelineType::input:
-      db::StreamInputs::setPlugins(new_list);
+      DbStreamInputs::setPlugins(new_list);
       break;
     case PipelineType::output:
-      db::StreamOutputs::setPlugins(new_list);
+      DbStreamOutputs::setPlugins(new_list);
       break;
   }
 
@@ -535,6 +556,8 @@ bool Manager::loadLocalPresetFile(const PipelineType& pipeline_type, const QStri
   if (!std::filesystem::exists(input_file)) {
     util::debug(std::format("Can't find the local preset \"{}\" on the filesystem", name.toStdString()));
 
+    update_used_presets_list(pipeline_type, "");
+
     return false;
   }
 
@@ -690,19 +713,19 @@ void Manager::set_last_preset_keys(const PipelineType& pipeline_type,
   if (package_name.isEmpty()) {
     switch (pipeline_type) {
       case PipelineType::input:
-        db::Main::setLastLoadedInputCommunityPackage(db::Main::defaultLastLoadedInputCommunityPackageValue());
+        DbMain::setLastLoadedInputCommunityPackage(DbMain::defaultLastLoadedInputCommunityPackageValue());
         break;
       case PipelineType::output:
-        db::Main::setLastLoadedOutputCommunityPackage(db::Main::defaultLastLoadedOutputCommunityPackageValue());
+        DbMain::setLastLoadedOutputCommunityPackage(DbMain::defaultLastLoadedOutputCommunityPackageValue());
         break;
     }
   } else {
     switch (pipeline_type) {
       case PipelineType::input:
-        db::Main::setLastLoadedInputCommunityPackage(package_name);
+        DbMain::setLastLoadedInputCommunityPackage(package_name);
         break;
       case PipelineType::output:
-        db::Main::setLastLoadedOutputCommunityPackage(package_name);
+        DbMain::setLastLoadedOutputCommunityPackage(package_name);
         break;
     }
   }
@@ -710,19 +733,19 @@ void Manager::set_last_preset_keys(const PipelineType& pipeline_type,
   if (preset_name.isEmpty()) {
     switch (pipeline_type) {
       case PipelineType::input:
-        db::Main::setLastLoadedInputPreset(db::Main::defaultLastLoadedInputPresetValue());
+        DbMain::setLastLoadedInputPreset(DbMain::defaultLastLoadedInputPresetValue());
         break;
       case PipelineType::output:
-        db::Main::setLastLoadedOutputPreset(db::Main::defaultLastLoadedOutputPresetValue());
+        DbMain::setLastLoadedOutputPreset(DbMain::defaultLastLoadedOutputPresetValue());
         break;
     }
   } else {
     switch (pipeline_type) {
       case PipelineType::input:
-        db::Main::setLastLoadedInputPreset(preset_name);
+        DbMain::setLastLoadedInputPreset(preset_name);
         break;
       case PipelineType::output:
-        db::Main::setLastLoadedOutputPreset(preset_name);
+        DbMain::setLastLoadedOutputPreset(preset_name);
         break;
     }
   }
@@ -812,56 +835,88 @@ auto Manager::create_wrapper(const PipelineType& pipeline_type, const QString& f
     -> std::optional<std::unique_ptr<PluginPresetBase>> {
   if (filter_name.startsWith(tags::plugin_name::BaseName::autogain)) {
     return std::make_unique<AutoGainPreset>(pipeline_type, filter_name.toStdString());
+
   } else if (filter_name.startsWith(tags::plugin_name::BaseName::bassEnhancer)) {
     return std::make_unique<BassEnhancerPreset>(pipeline_type, filter_name.toStdString());
+
   } else if (filter_name.startsWith(tags::plugin_name::BaseName::bassLoudness)) {
     return std::make_unique<BassLoudnessPreset>(pipeline_type, filter_name.toStdString());
+
   } else if (filter_name.startsWith(tags::plugin_name::BaseName::compressor)) {
     return std::make_unique<CompressorPreset>(pipeline_type, filter_name.toStdString());
+
   } else if (filter_name.startsWith(tags::plugin_name::BaseName::convolver)) {
     return std::make_unique<ConvolverPreset>(pipeline_type, filter_name.toStdString());
+
   } else if (filter_name.startsWith(tags::plugin_name::BaseName::crossfeed)) {
     return std::make_unique<CrossfeedPreset>(pipeline_type, filter_name.toStdString());
+
+  } else if (filter_name.startsWith(tags::plugin_name::BaseName::crusher)) {
+    return std::make_unique<CrusherPreset>(pipeline_type, filter_name.toStdString());
+
   } else if (filter_name.startsWith(tags::plugin_name::BaseName::crystalizer)) {
     return std::make_unique<CrystalizerPreset>(pipeline_type, filter_name.toStdString());
+
   } else if (filter_name.startsWith(tags::plugin_name::BaseName::deesser)) {
     return std::make_unique<DeesserPreset>(pipeline_type, filter_name.toStdString());
+
   } else if (filter_name.startsWith(tags::plugin_name::BaseName::delay)) {
     return std::make_unique<DelayPreset>(pipeline_type, filter_name.toStdString());
+
   } else if (filter_name.startsWith(tags::plugin_name::BaseName::deepfilternet)) {
     return std::make_unique<DeepFilterNetPreset>(pipeline_type, filter_name.toStdString());
+
   } else if (filter_name.startsWith(tags::plugin_name::BaseName::echoCanceller)) {
     return std::make_unique<EchoCancellerPreset>(pipeline_type, filter_name.toStdString());
+
   } else if (filter_name.startsWith(tags::plugin_name::BaseName::equalizer)) {
     return std::make_unique<EqualizerPreset>(pipeline_type, filter_name.toStdString());
+
   } else if (filter_name.startsWith(tags::plugin_name::BaseName::exciter)) {
     return std::make_unique<ExciterPreset>(pipeline_type, filter_name.toStdString());
+
   } else if (filter_name.startsWith(tags::plugin_name::BaseName::expander)) {
     return std::make_unique<ExpanderPreset>(pipeline_type, filter_name.toStdString());
+
   } else if (filter_name.startsWith(tags::plugin_name::BaseName::filter)) {
     return std::make_unique<FilterPreset>(pipeline_type, filter_name.toStdString());
+
   } else if (filter_name.startsWith(tags::plugin_name::BaseName::gate)) {
     return std::make_unique<GatePreset>(pipeline_type, filter_name.toStdString());
+
+  } else if (filter_name.startsWith(tags::plugin_name::BaseName::voiceSuppressor)) {
+    return std::make_unique<VoiceSuppressorPreset>(pipeline_type, filter_name.toStdString());
+
   } else if (filter_name.startsWith(tags::plugin_name::BaseName::levelMeter)) {
     return std::make_unique<LevelMeterPreset>(pipeline_type, filter_name.toStdString());
+
   } else if (filter_name.startsWith(tags::plugin_name::BaseName::limiter)) {
     return std::make_unique<LimiterPreset>(pipeline_type, filter_name.toStdString());
+
   } else if (filter_name.startsWith(tags::plugin_name::BaseName::loudness)) {
     return std::make_unique<LoudnessPreset>(pipeline_type, filter_name.toStdString());
+
   } else if (filter_name.startsWith(tags::plugin_name::BaseName::maximizer)) {
     return std::make_unique<MaximizerPreset>(pipeline_type, filter_name.toStdString());
+
   } else if (filter_name.startsWith(tags::plugin_name::BaseName::multibandCompressor)) {
     return std::make_unique<MultibandCompressorPreset>(pipeline_type, filter_name.toStdString());
+
   } else if (filter_name.startsWith(tags::plugin_name::BaseName::multibandGate)) {
     return std::make_unique<MultibandGatePreset>(pipeline_type, filter_name.toStdString());
+
   } else if (filter_name.startsWith(tags::plugin_name::BaseName::pitch)) {
     return std::make_unique<PitchPreset>(pipeline_type, filter_name.toStdString());
+
   } else if (filter_name.startsWith(tags::plugin_name::BaseName::reverb)) {
     return std::make_unique<ReverbPreset>(pipeline_type, filter_name.toStdString());
+
   } else if (filter_name.startsWith(tags::plugin_name::BaseName::rnnoise)) {
     return std::make_unique<RNNoisePreset>(pipeline_type, filter_name.toStdString());
+
   } else if (filter_name.startsWith(tags::plugin_name::BaseName::speex)) {
     return std::make_unique<SpeexPreset>(pipeline_type, filter_name.toStdString());
+
   } else if (filter_name.startsWith(tags::plugin_name::BaseName::stereoTools)) {
     return std::make_unique<StereoToolsPreset>(pipeline_type, filter_name.toStdString());
   }
@@ -875,13 +930,16 @@ void Manager::update_used_presets_list(const PipelineType& pipeline_type, const
   QList<QString> names;
   QList<int> count_list;
 
-  names = (pipeline_type == PipelineType::input) ? db::StreamInputs::usedPresets() : db::StreamOutputs::usedPresets();
+  names = (pipeline_type == PipelineType::input) ? DbStreamInputs::usedPresets() : DbStreamOutputs::usedPresets();
 
   // removing from the list presets that are not installed anymore
 
-  names.removeIf([&](const QString& s) {
-    return std::ranges::none_of(dir_manager.getLocalPresetsPaths(pipeline_type),
-                                [&](const auto p) { return s.startsWith(QString::fromStdString(p.stem().string())); });
+  names.removeIf([&](const QString& name_and_count) {
+    return std::ranges::none_of(dir_manager.getLocalPresetsPaths(pipeline_type), [&](const auto p) {
+      auto name = name_and_count.split(":")[0];
+
+      return name == p.stem().string();
+    });
   });
 
   bool contains_name = false;
@@ -910,9 +968,9 @@ void Manager::update_used_presets_list(const PipelineType& pipeline_type, const
   }
 
   if (pipeline_type == PipelineType::input) {
-    db::StreamInputs::setUsedPresets(names);
+    DbStreamInputs::setUsedPresets(names);
   } else {
-    db::StreamOutputs::setUsedPresets(names);
+    DbStreamOutputs::setUsedPresets(names);
   }
 
   std::multimap<int, QString> usageMap;
@@ -933,15 +991,15 @@ void Manager::update_used_presets_list(const PipelineType& pipeline_type, const
     sortedList << it.second;
   }
 
-  if (sortedList.size() >
-      db::Main::maxMostUsedPresets()) {  // We can't have many entries in the tray menu. There is no space for that.
-    sortedList.resize(db::Main::maxMostUsedPresets());
+  if (sortedList.size() > DbMain::maxMostUsedPresets()) {
+    // We can't have many entries in the tray menu. There is no space for that.
+    sortedList.resize(DbMain::maxMostUsedPresets());
   }
 
   if (pipeline_type == PipelineType::input) {
-    db::StreamInputs::setMostUsedPresets(sortedList);
+    DbStreamInputs::setMostUsedPresets(sortedList);
   } else {
-    db::StreamOutputs::setMostUsedPresets(sortedList);
+    DbStreamOutputs::setMostUsedPresets(sortedList);
   }
 }
 
