@@ -1,5 +1,5 @@
 /**
- * Copyright © 2017-2025 Wellington Wallace
+ * Copyright © 2017-2026 Wellington Wallace
  *
  * This file is part of Easy Effects.
  *
@@ -246,13 +246,17 @@ Manager::Manager()
       }
 
       for (auto& node : nodes) {
-        if (node.media_class == tags::pipewire::media_class::source) {
-          node.device_route_name = device.input_route_name;
-          node.device_route_description = device.input_route_description;
+        if (node.media_class == tags::pipewire::media_class::source ||
+            node.media_class == tags::pipewire::media_class::virtual_source) {
+          if (node.device_route_name != device.input_route_name ||
+              node.device_route_description != device.input_route_description) {
+            node.device_route_name = device.input_route_name;
+            node.device_route_description = device.input_route_description;
+
+            Q_EMIT sourceRouteChanged(node);
+          }
 
           model_nodes.update_info(node);
-
-          Q_EMIT sourceRouteChanged(node);
         }
       }
     });
@@ -268,13 +272,17 @@ Manager::Manager()
       }
 
       for (auto& node : nodes) {
-        if (node.media_class == tags::pipewire::media_class::sink) {
-          node.device_route_name = device.output_route_name;
-          node.device_route_description = device.output_route_description;
+        if (node.media_class == tags::pipewire::media_class::sink ||
+            node.media_class == tags::pipewire::media_class::virtual_sink) {
+          if (node.device_route_name != device.output_route_name ||
+              node.device_route_description != device.output_route_description) {
+            node.device_route_name = device.output_route_name;
+            node.device_route_description = device.output_route_description;
+
+            Q_EMIT sinkRouteChanged(node);
+          }
 
           model_nodes.update_info(node);
-
-          Q_EMIT sinkRouteChanged(node);
         }
       }
     });
