@@ -1,5 +1,5 @@
 /**
- * Copyright © 2017-2025 Wellington Wallace
+ * Copyright © 2017-2026 Wellington Wallace
  *
  * This file is part of Easy Effects.
  *
@@ -100,7 +100,9 @@ Nodes::Nodes(QObject* parent)
     proxy_source_devices.setFilterRole(static_cast<int>(Roles::MediaClass));
     proxy_source_devices.setDynamicSortFilter(true);
 
-    auto pattern = "^" + QString(tags::pipewire::media_class::source) + "$";
+    auto pattern = "^(" + QString(tags::pipewire::media_class::source) + "|" +
+                   QString(tags::pipewire::media_class::virtual_source) + ")$";
+
     proxy_source_devices.setFilterRegularExpression(
         QRegularExpression(pattern, QRegularExpression::CaseInsensitiveOption));
 
@@ -120,7 +122,9 @@ Nodes::Nodes(QObject* parent)
     proxy_sink_devices.setFilterRole(static_cast<int>(Roles::MediaClass));
     proxy_sink_devices.setDynamicSortFilter(true);
 
-    auto pattern = "^" + QString(tags::pipewire::media_class::sink) + "$";
+    auto pattern = "^(" + QString(tags::pipewire::media_class::sink) + "|" +
+                   QString(tags::pipewire::media_class::virtual_sink) + ")$";
+
     proxy_sink_devices.setFilterRegularExpression(
         QRegularExpression(pattern, QRegularExpression::CaseInsensitiveOption));
 
@@ -132,56 +136,20 @@ Nodes::Nodes(QObject* parent)
   }
 
   connect(
-      db::Manager::self().streamOutputs, &db::StreamOutputs::blocklistChanged, this,
-      [this]() {
-        const auto blocklist = db::StreamOutputs::blocklist();
-
-        for (qsizetype n = 0; n < list.size(); n++) {
-          if (list[n].media_class != tags::pipewire::media_class::output_stream) {
-            continue;
-          }
-
-          if (blocklist.contains(list[n].name) || blocklist.contains(list[n].application_id) ||
-              blocklist.contains(list[n].app_process_binary)) {
-            update_field(n, Roles::IsBlocklisted, true);
-
-            pw::Manager::self().disconnectStream(list[n].id);
-          } else {
-            update_field(n, Roles::IsBlocklisted, false);
-
-            if (db::Main::processAllOutputs()) {
-              pw::Manager::self().connectStreamOutput(list[n].id);
-            }
-          }
-        }
-      },
+      DbStreamOutputs::self(), &DbStreamOutputs::blocklistChanged, this, [&]() { onOutputBlocklistChanged(); },
       Qt::QueuedConnection);
 
   connect(
-      db::Manager::self().streamInputs, &db::StreamInputs::blocklistChanged, this,
-      [this]() {
-        const auto blocklist = db::StreamInputs::blocklist();
+      DbStreamOutputs::self(), &DbStreamOutputs::blocklistUsesMediaNameChanged, this,
+      [&]() { onOutputBlocklistChanged(); }, Qt::QueuedConnection);
 
-        for (qsizetype n = 0; n < list.size(); n++) {
-          if (list[n].media_class != tags::pipewire::media_class::input_stream) {
-            continue;
-          }
-
-          if (blocklist.contains(list[n].name) || blocklist.contains(list[n].application_id) ||
-              blocklist.contains(list[n].app_process_binary)) {
-            update_field(n, Roles::IsBlocklisted, true);
-
-            pw::Manager::self().disconnectStream(list[n].id);
-          } else {
-            update_field(n, Roles::IsBlocklisted, false);
-
-            if (db::Main::processAllInputs()) {
-              pw::Manager::self().connectStreamInput(list[n].id);
-            }
-          }
-        }
-      },
+  connect(
+      DbStreamInputs::self(), &DbStreamInputs::blocklistChanged, this, [&]() { onInputBlocklistChanged(); },
       Qt::QueuedConnection);
+
+  connect(
+      DbStreamInputs::self(), &DbStreamInputs::blocklistUsesMediaNameChanged, this,
+      [&]() { onInputBlocklistChanged(); }, Qt::QueuedConnection);
 }
 
 int Nodes::rowCount(const QModelIndex& /* parent */) const {
@@ -307,45 +275,59 @@ bool Nodes::setData(const QModelIndex& index, const QVariant& value, int role) {
 
         if (it->is_blocklisted) {
           if (it->media_class == tags::pipewire::media_class::output_stream) {
-            auto blocklist = db::StreamOutputs::blocklist();
+            auto blocklist = DbStreamOutputs::blocklist();
 
             if (blocklist.indexOf(it->name) == -1) {
-              blocklist.append(it->name);
-              db::StreamOutputs::setBlocklist(blocklist);
+              if (DbStreamOutputs::blocklistUsesMediaName()) {
+                blocklist.append(it->name + ":" + it->media_name);
+              } else {
+                blocklist.append(it->name);
+              }
+
+              DbStreamOutputs::setBlocklist(blocklist);
             }
 
           } else if (it->media_class == tags::pipewire::media_class::input_stream) {
-            auto blocklist = db::StreamInputs::blocklist();
+            auto blocklist = DbStreamInputs::blocklist();
 
             if (blocklist.indexOf(it->name) == -1) {
-              blocklist.append(it->name);
-              db::StreamInputs::setBlocklist(blocklist);
+              if (DbStreamInputs::blocklistUsesMediaName()) {
+                blocklist.append(it->name + ":" + it->media_name);
+              } else {
+                blocklist.append(it->name);
+              }
+
+              DbStreamInputs::setBlocklist(blocklist);
             }
           }
         } else {
           if (it->media_class == tags::pipewire::media_class::output_stream) {
-            auto blocklist = db::StreamOutputs::blocklist();
+            auto blocklist = DbStreamOutputs::blocklist();
 
             auto idx = blocklist.indexOf(it->name);
 
             idx = (idx == -1) ? blocklist.indexOf(it->application_id) : idx;
 
+            idx = (idx == -1) ? blocklist.indexOf(it->name + ":" + it->media_name) : idx;
+
             if (idx != -1) {
               blocklist.removeAt(idx);
 
-              db::StreamOutputs::setBlocklist(blocklist);
+              DbStreamOutputs::setBlocklist(blocklist);
             }
           } else if (it->media_class == tags::pipewire::media_class::input_stream) {
-            auto blocklist = db::StreamInputs::blocklist();
+            auto blocklist = DbStreamInputs::blocklist();
 
             auto idx = blocklist.indexOf(it->name);
 
             idx = (idx == -1) ? blocklist.indexOf(it->application_id) : idx;
 
+            idx = (idx == -1) ? blocklist.indexOf(it->name + ":" + it->media_name) : idx;
+
             if (idx != -1) {
               blocklist.removeAt(idx);
 
-              db::StreamInputs::setBlocklist(blocklist);
+              DbStreamInputs::setBlocklist(blocklist);
             }
           }
         }
@@ -366,7 +348,7 @@ auto Nodes::get_list() -> QList<NodeInfo> {
   return list;
 }
 
-void Nodes::append(const NodeInfo& info) {
+void Nodes::append(NodeInfo info) {
   beginInsertRows(QModelIndex(), list.size(), list.size());
 
   list.append(info);
@@ -432,14 +414,14 @@ auto Nodes::has_serial(const uint& serial) -> bool {
   return std::ranges::find_if(list, [=](const NodeInfo& ni) { return ni.serial == serial; }) != list.end();
 }
 
-void Nodes::update_info(const NodeInfo& new_info) {
+void Nodes::update_info(NodeInfo new_info) {
   const int row = get_row_by_serial(new_info.serial);
 
   if (row < 0) {
     return;
   }
 
-  NodeInfo& info = list[row];
+  NodeInfo info = list[row];
   bool anyChanged = false;
 
   auto updateIfDifferent = [&](auto role, const auto& oldVal, const auto& newVal) {
@@ -575,7 +557,7 @@ QString Nodes::getNodeName(const uint& rowIndex) {
   return list[rowIndex].name;
 }
 
-QString Nodes::getNodeDescription(const QString& nodeName) {
+QString Nodes::getNodeDescription(QString nodeName) {
   for (const auto& node : list) {
     if (node.name == nodeName) {
       return node.description;
@@ -585,7 +567,7 @@ QString Nodes::getNodeDescription(const QString& nodeName) {
   return "";
 }
 
-QModelIndex Nodes::getModelIndexByName(const QString& nodeName) {
+QModelIndex Nodes::getModelIndexByName(QString nodeName) {
   for (int n = 0; n < list.size(); n++) {
     if (list[n].name == nodeName) {
       return this->index(n);
@@ -595,7 +577,7 @@ QModelIndex Nodes::getModelIndexByName(const QString& nodeName) {
   return this->index(-1);
 }
 
-auto Nodes::get_node_by_name(const QString& name) -> NodeInfo {
+auto Nodes::get_node_by_name(QString name) -> NodeInfo {
   for (const auto& node : list) {
     if (node.name == name) {
       return node;
@@ -627,4 +609,62 @@ auto Nodes::get_nodes_by_device_id(const uint& id) -> QList<NodeInfo> {
   return nodes;
 }
 
+void Nodes::onOutputBlocklistChanged() {
+  const auto blocklist = DbStreamOutputs::blocklist();
+
+  for (qsizetype n = 0; n < list.size(); n++) {
+    if (list[n].media_class != tags::pipewire::media_class::output_stream) {
+      continue;
+    }
+
+    auto is_blocklisted = blocklist.contains(list[n].name) || blocklist.contains(list[n].application_id) ||
+                          blocklist.contains(list[n].app_process_binary);
+
+    if (DbStreamOutputs::blocklistUsesMediaName()) {
+      is_blocklisted = is_blocklisted || blocklist.contains(list[n].name + ":" + list[n].media_name);
+    }
+
+    if (is_blocklisted) {
+      update_field(n, Roles::IsBlocklisted, true);
+
+      pw::Manager::self().disconnectStream(list[n].id);
+    } else {
+      update_field(n, Roles::IsBlocklisted, false);
+
+      if (DbMain::processAllOutputs()) {
+        pw::Manager::self().connectStreamOutput(list[n].id);
+      }
+    }
+  }
+}
+
+void Nodes::onInputBlocklistChanged() {
+  const auto blocklist = DbStreamInputs::blocklist();
+
+  for (qsizetype n = 0; n < list.size(); n++) {
+    if (list[n].media_class != tags::pipewire::media_class::input_stream) {
+      continue;
+    }
+
+    auto is_blocklisted = blocklist.contains(list[n].name) || blocklist.contains(list[n].application_id) ||
+                          blocklist.contains(list[n].app_process_binary);
+
+    if (DbStreamInputs::blocklistUsesMediaName()) {
+      is_blocklisted = is_blocklisted || blocklist.contains(list[n].name + ":" + list[n].media_name);
+    }
+
+    if (is_blocklisted) {
+      update_field(n, Roles::IsBlocklisted, true);
+
+      pw::Manager::self().disconnectStream(list[n].id);
+    } else {
+      update_field(n, Roles::IsBlocklisted, false);
+
+      if (DbMain::processAllInputs()) {
+        pw::Manager::self().connectStreamInput(list[n].id);
+      }
+    }
+  }
+}
+
 }  // namespace pw::models
