@@ -1,5 +1,5 @@
 /**
- * Copyright © 2017-2025 Wellington Wallace
+ * Copyright © 2017-2026 Wellington Wallace
  *
  * This file is part of Easy Effects.
  *
@@ -19,7 +19,9 @@
 
 #include "rnnoise.hpp"
 #include <qstandardpaths.h>
+#include <qtmetamacros.h>
 #include <algorithm>
+#include <cstdio>
 #include <filesystem>
 #include <format>
 #include "easyeffects_db_rnnoise.h"
@@ -30,7 +32,6 @@
 #endif
 #include <sys/types.h>
 #include <cmath>
-#include <cstdio>
 #include <memory>
 #include <mutex>
 #include <span>
@@ -112,6 +113,10 @@ RNNoise::RNNoise(const std::string& tag, pw::Manager* pipe_manager, PipelineType
   util::warning("The RNNoise library was not available at compilation time. The noise reduction filter won't work");
 
   settings->setEnableVad(false);
+
+  packageInstalled = false;
+
+  Q_EMIT packageInstalledChanged();
 #endif
 }
 
@@ -120,7 +125,6 @@ RNNoise::~RNNoise() {
     disconnect_from_pw();
   }
 
-  disconnect();
   settings->disconnect();
 
   std::scoped_lock<std::mutex> lock(data_mutex);
@@ -138,7 +142,16 @@ void RNNoise::reset() {
   settings->setDefaults();
 }
 
+void RNNoise::clear_data() {
+  setup();
+}
+
 void RNNoise::setup() {
+  if (rate == 0 || n_samples == 0) {
+    // Some signals may be emitted before PipeWire calls our setup function
+    return;
+  }
+
   std::scoped_lock<std::mutex> lock(data_mutex);
 
   resampler_ready = false;
@@ -168,21 +181,35 @@ void RNNoise::process(std::span<float>& left_in,
                       std::span<float>& right_out) {
   std::scoped_lock<std::mutex> lock(data_mutex);
 
-  if (bypass || !rnnoise_ready) {
+  if (bypass) {
     std::ranges::copy(left_in, left_out.begin());
     std::ranges::copy(right_in, right_out.begin());
 
     return;
   }
 
+#ifdef ENABLE_RNNOISE
+  if (!rnnoise_ready) {
+    std::ranges::fill(left_out, 0.0F);
+    std::ranges::fill(right_out, 0.0F);
+
+    return;
+  }
+#else
+  std::ranges::copy(left_in, left_out.begin());
+  std::ranges::copy(right_in, right_out.begin());
+
+  return;
+#endif
+
   if (input_gain != 1.0F) {
     apply_gain(left_in, right_in, input_gain);
   }
 
   if (resample) {
     if (resampler_ready) {
-      const auto resampled_inL = resampler_inL->process(left_in, false);
-      const auto resampled_inR = resampler_inR->process(right_in, false);
+      const auto resampled_inL = resampler_inL->process(left_in);
+      const auto resampled_inR = resampler_inR->process(right_in);
 
       resampled_data_L.resize(0U);
       resampled_data_R.resize(0U);
@@ -191,8 +218,8 @@ void RNNoise::process(std::span<float>& left_in,
       remove_noise(resampled_inL, resampled_inR, resampled_data_L, resampled_data_R);
 #endif
 
-      auto resampled_outL = resampler_outL->process(resampled_data_L, false);
-      auto resampled_outR = resampler_outR->process(resampled_data_R, false);
+      auto resampled_outL = resampler_outL->process(resampled_data_L);
+      auto resampled_outR = resampler_outR->process(resampled_data_R);
 
       buf_out_L.insert(buf_out_L.end(), resampled_outL.begin(), resampled_outL.end());
       buf_out_R.insert(buf_out_R.end(), resampled_outR.begin(), resampled_outR.end());
@@ -210,7 +237,7 @@ void RNNoise::process(std::span<float>& left_in,
     util::copy_bulk(buf_out_L, left_out);
     util::copy_bulk(buf_out_R, right_out);
   } else {
-    const uint offset = 2U * (left_out.size() - buf_out_L.size());
+    const uint offset = left_out.size() - buf_out_L.size();
 
     if (offset != latency_n_frames) {
       latency_n_frames = offset;
@@ -259,8 +286,8 @@ auto RNNoise::search_model_path(const std::string& name) -> std::string {
   // Given the model name without extension, search the full path on the filesystem.
   const auto model_filename = name + rnnn_ext;
 
-  const auto community_package = (pipeline_type == PipelineType::input) ? db::Main::lastLoadedInputCommunityPackage()
-                                                                        : db::Main::lastLoadedOutputCommunityPackage();
+  const auto community_package = (pipeline_type == PipelineType::input) ? DbMain::lastLoadedInputCommunityPackage()
+                                                                        : DbMain::lastLoadedOutputCommunityPackage();
 
   std::string model_full_path;
 
@@ -325,10 +352,21 @@ auto RNNoise::get_model_from_name() -> RNNModel* {
 
   RNNModel* m = nullptr;
 
-  if (FILE* f = fopen(path.c_str(), "r"); f != nullptr) {
-    m = rnnoise_model_from_file(f);
+  if (model_file != nullptr) {
+    fclose(model_file);
+  }
 
-    fclose(f);
+  /**
+   * We prefer using "rnnoise_model_from_file" because it's more robust than
+   * "rnnoise_model_from_filename". Indeed, when an invalid model is loaded, it
+   * does not crash and automatically switches to the Standard Model.
+   *
+   * Note that from RNNoise v0.1.1 a new binary model format is used, so the old
+   * format may not work even if the file is correctly loaded (the signal does
+   * not change like in a passthrough mode). See issue #4748.
+   */
+  if (model_file = fopen(path.c_str(), "rb"); model_file != nullptr) {
+    m = rnnoise_model_from_file(model_file);
   }
 
   standard_model = (m == nullptr);
@@ -380,9 +418,14 @@ void RNNoise::free_rnnoise() {
     rnnoise_model_free(model);
   }
 
+  if (model_file != nullptr) {
+    fclose(model_file);
+  }
+
   state_left = nullptr;
   state_right = nullptr;
   model = nullptr;
+  model_file = nullptr;
 }
 
 #endif
