@@ -1,5 +1,5 @@
 /**
- * Copyright © 2017-2025 Wellington Wallace
+ * Copyright © 2017-2026 Wellington Wallace
  *
  * This file is part of Easy Effects.
  *
@@ -20,7 +20,10 @@
 #include "spectrum.hpp"
 #include <fftw3.h>
 #include <qlist.h>
+#include <qnamespace.h>
+#include <qobjectdefs.h>
 #include <qtypes.h>
+#include <QApplication>
 #include <QString>
 #include <algorithm>
 #include <cassert>
@@ -29,6 +32,7 @@
 #include <cstring>
 #include <format>
 #include <memory>
+#include <mutex>
 #include <numbers>
 #include <span>
 #include <string>
@@ -44,9 +48,8 @@
 
 Spectrum::Spectrum(const std::string& tag, pw::Manager* pipe_manager, PipelineType pipe_type, QString instance_id)
     : PluginBase(tag, "spectrum", tags::plugin_package::Package::ee, instance_id, pipe_manager, pipe_type),
-      settings(db::Spectrum::self()),
-      fftw_ready(true) {
-  bypass = !db::Spectrum::state();
+      settings(DbSpectrum::self()) {
+  bypass = !DbSpectrum::state();
   // Precompute the Hann window, which is an expensive operation.
   // https://en.wikipedia.org/wiki/Hann_function
   for (size_t n = 0; n < n_bands; n++) {
@@ -59,6 +62,10 @@ Spectrum::Spectrum(const std::string& tag, pw::Manager* pipe_manager, PipelineTy
 
   plan = fftwf_plan_dft_r2c_1d(static_cast<int>(n_bands), real_input.data(), complex_output, FFTW_ESTIMATE);
 
+  if (plan != nullptr && complex_output != nullptr) {
+    fftw_ready = true;
+  }
+
   const auto lv2_plugin_uri = "http://lsp-plug.in/plugins/lv2/comp_delay_x2_stereo";
 
   lv2_wrapper = std::make_unique<lv2::Lv2Wrapper>(lv2_plugin_uri);
@@ -79,18 +86,19 @@ Spectrum::Spectrum(const std::string& tag, pw::Manager* pipe_manager, PipelineTy
   lv2_wrapper->set_control_port_value("wet_l", util::db_to_linear(0.0F));
   lv2_wrapper->set_control_port_value("wet_r", util::db_to_linear(0.0F));
 
-  BIND_LV2_PORT("time_l", avsyncDelay, setAvsyncDelay, db::Spectrum::avsyncDelayChanged);
-  BIND_LV2_PORT("time_r", avsyncDelay, setAvsyncDelay, db::Spectrum::avsyncDelayChanged);
+  BIND_LV2_PORT("time_l", avsyncDelay, setAvsyncDelay, DbSpectrum::avsyncDelayChanged);
+  BIND_LV2_PORT("time_r", avsyncDelay, setAvsyncDelay, DbSpectrum::avsyncDelayChanged);
 
-  connect(db::Spectrum::self(), &db::Spectrum::stateChanged, [&]() { bypass = !db::Spectrum::state(); });
+  connect(DbSpectrum::self(), &DbSpectrum::stateChanged, [&]() { bypass = !DbSpectrum::state(); });
 }
 
 Spectrum::~Spectrum() {
+  std::scoped_lock<std::mutex> lock(data_mutex);
+
   if (connected_to_pw) {
     disconnect_from_pw();
   }
 
-  disconnect();
   settings->disconnect();
 
   fftw_ready = false;
@@ -106,7 +114,28 @@ Spectrum::~Spectrum() {
 
 void Spectrum::reset() {}
 
+void Spectrum::clear_data() {
+  if (lv2_wrapper == nullptr) {
+    return;
+  }
+
+  {
+    std::scoped_lock<std::mutex> lock(data_mutex);
+
+    lv2_wrapper->destroy_instance();
+  }
+
+  setup();
+}
+
 void Spectrum::setup() {
+  if (rate == 0 || n_samples == 0) {
+    // Some signals may be emitted before PipeWire calls our setup function
+    return;
+  }
+
+  std::scoped_lock<std::mutex> lock(data_mutex);
+
   std::ranges::fill(real_input, 0.0F);
   std::ranges::fill(latest_samples_mono, 0.0F);
 
@@ -122,10 +151,22 @@ void Spectrum::setup() {
     return;
   }
 
-  if (lv2_wrapper->get_rate() != rate) {
-    util::debug(std::format("{} creating instance of comp delay x2 stereo for spectrum A/V sync", log_tag));
-    lv2_wrapper->create_instance(rate);
-  }
+  ready = false;
+
+  // NOLINTBEGIN(clang-analyzer-cplusplus.NewDeleteLeaks)
+  QMetaObject::invokeMethod(
+      baseWorker,
+      [this] {
+        util::debug(std::format("{} creating instance of comp delay x2 stereo for spectrum A/V sync", log_tag));
+
+        lv2_wrapper->create_instance(rate);
+
+        std::scoped_lock<std::mutex> lock(data_mutex);
+
+        ready = true;
+      },
+      Qt::QueuedConnection);
+  // NOLINTEND(clang-analyzer-cplusplus.NewDeleteLeaks)
 }
 
 void Spectrum::process(std::span<float>& left_in,
@@ -135,7 +176,7 @@ void Spectrum::process(std::span<float>& left_in,
   std::ranges::copy(left_in, left_out.begin());
   std::ranges::copy(right_in, right_out.begin());
 
-  if (bypass || !fftw_ready) {
+  if (bypass || !fftw_ready || !ready) {
     return;
   }
 
@@ -240,10 +281,12 @@ void Spectrum::process(std::span<float>& left_in,
 }
 
 auto Spectrum::compute_magnitudes() -> std::tuple<uint, QList<double>> {
+  std::scoped_lock<std::mutex> lock(data_mutex);
+
   // Early return if no new data is available, ie if process() has not been
   // called since our last compute_magnitudes() call.
   int curr_control = db_control.load();
-  if (!(curr_control & static_cast<int>(DB_BIT::NEWDATA))) {
+  if (!fftw_ready || !(curr_control & static_cast<int>(DB_BIT::NEWDATA))) {
     return {0, {}};
   }
 
