@@ -1,5 +1,5 @@
 /**
- * Copyright © 2017-2025 Wellington Wallace
+ * Copyright © 2017-2026 Wellington Wallace
  *
  * This file is part of Easy Effects.
  *
@@ -53,9 +53,9 @@ StreamInputEffects::StreamInputEffects(pw::Manager* pipe_manager) : EffectsBase(
   connect(
       pm, &pw::Manager::sourceAdded, this,
       [&](pw::NodeInfo node) {
-        if (node.name == db::StreamInputs::inputDevice()) {
-          if (db::Main::bypass()) {
-            db::Main::setBypass(false);
+        if (node.name == DbStreamInputs::inputDevice()) {
+          if (DbMain::bypass()) {
+            DbMain::setBypass(false);
 
             return;  // filter connected through update_bypass_state
           }
@@ -68,35 +68,17 @@ StreamInputEffects::StreamInputEffects(pw::Manager* pipe_manager) : EffectsBase(
       Qt::QueuedConnection);
 
   connect(
-      pm, &pw::Manager::newDefaultSourceName, this,
-      [](QString name) {
-        if (db::StreamInputs::useDefaultInputDevice() || db::StreamInputs::inputDevice().isEmpty()) {
-          db::StreamInputs::setInputDevice(name);
-        }
-      },
-      Qt::QueuedConnection);
-
-  connect(
-      db::StreamInputs::self(), &db::StreamInputs::useDefaultInputDeviceChanged, this,
+      DbStreamInputs::self(), &DbStreamInputs::inputDeviceChanged, this,
       [&]() {
-        if (db::StreamInputs::useDefaultInputDevice()) {
-          db::StreamInputs::setInputDevice(pm->defaultInputDeviceName);
-        }
-      },
-      Qt::QueuedConnection);
-
-  connect(
-      db::StreamInputs::self(), &db::StreamInputs::inputDeviceChanged, this,
-      [&]() {
-        const auto name = db::StreamInputs::inputDevice();
+        const auto name = DbStreamInputs::inputDevice();
 
         if (name.isEmpty()) {
           return;
         }
 
         if (auto node = pm->model_nodes.get_node_by_name(name); node.serial != SPA_ID_INVALID) {
-          if (db::Main::bypass()) {
-            db::Main::setBypass(false);
+          if (DbMain::bypass()) {
+            DbMain::setBypass(false);
 
             return;  // filter connected through update_bypass_state
           }
@@ -109,10 +91,10 @@ StreamInputEffects::StreamInputEffects(pw::Manager* pipe_manager) : EffectsBase(
       Qt::QueuedConnection);
 
   connect(
-      db::StreamInputs::self(), &db::StreamInputs::pluginsChanged, this,
+      DbStreamInputs::self(), &DbStreamInputs::pluginsChanged, this,
       [&]() {
-        if (db::Main::bypass()) {
-          db::Main::setBypass(false);
+        if (DbMain::bypass()) {
+          DbMain::setBypass(false);
 
           return;  // filter connected through update_bypass_state
         }
@@ -128,35 +110,31 @@ StreamInputEffects::StreamInputEffects(pw::Manager* pipe_manager) : EffectsBase(
   connect(
       pm, &pw::Manager::sourceRouteChanged, this,
       [](pw::NodeInfo node) {
-        if (node.name == db::StreamInputs::inputDevice()) {
+        if (node.name == DbStreamInputs::inputDevice()) {
           presets::Manager::self().autoload(PipelineType::input, node.name, node.device_route_description);
         }
       },
       Qt::QueuedConnection);
 
   connect(
-      db::StreamInputs::self(), &db::StreamInputs::listenToMicChanged, this,
-      [&]() { set_listen_to_mic(db::StreamInputs::listenToMic()); }, Qt::QueuedConnection);
+      DbStreamInputs::self(), &DbStreamInputs::listenToMicChanged, this,
+      [&]() { set_listen_to_mic(DbStreamInputs::listenToMic()); }, Qt::QueuedConnection);
 
   auto* PULSE_SOURCE = std::getenv("PULSE_SOURCE");
 
   if (PULSE_SOURCE != nullptr && PULSE_SOURCE != tags::pipewire::ee_source_name) {
     auto node = pm->model_nodes.get_node_by_name(PULSE_SOURCE);
 
-    db::StreamInputs::setInputDevice(PULSE_SOURCE);
+    DbStreamInputs::setInputDevice(PULSE_SOURCE);
   }
 
-  if (db::StreamInputs::inputDevice().isEmpty()) {
-    db::StreamInputs::setInputDevice(pm->defaultInputDeviceName);
+  if (DbStreamInputs::inputDevice().isEmpty()) {
+    DbStreamInputs::setInputDevice(pm->defaultInputDeviceName);
   }
 
   connect_filters();
 
-  if (auto node = pm->model_nodes.get_node_by_name(db::StreamInputs::inputDevice()); node.serial != SPA_ID_INVALID) {
-    presets::Manager::self().autoload(PipelineType::input, node.name, node.device_route_description);
-  }
-
-  set_listen_to_mic(db::StreamInputs::listenToMic());
+  set_listen_to_mic(DbStreamInputs::listenToMic());
 }
 
 StreamInputEffects::~StreamInputEffects() {
@@ -194,12 +172,12 @@ void StreamInputEffects::on_link_changed(const pw::LinkInfo link_info) {
       util::debug("At least one app linked to our device wants to play. Linking our filters.");
 
       connect_filters();
-    };
+    }
   } else {
-    if (db::Main::inactivityTimerEnable()) {
+    if (DbMain::inactivityTimerEnable()) {
       // if the timer is enabled, wait for the timeout, then unlink plugin pipeline
 
-      QTimer::singleShot(db::Main::inactivityTimeout() * 1000, this, [&]() {
+      QTimer::singleShot(DbMain::inactivityTimeout() * 1000, this, [&]() {
         if (!apps_want_to_play() && !list_proxies.empty()) {
           util::debug("No app linked to our device wants to play. Unlinking our filters.");
 
@@ -219,22 +197,22 @@ void StreamInputEffects::on_link_changed(const pw::LinkInfo link_info) {
 void StreamInputEffects::connect_filters(const bool& bypass) {
   // checking if the output device exists
 
-  if (db::StreamInputs::inputDevice().isEmpty()) {
+  if (DbStreamInputs::inputDevice().isEmpty()) {
     util::debug("No input device set. Aborting the link");
 
     return;
   }
 
-  auto input_device = pm->model_nodes.get_node_by_name(db::StreamInputs::inputDevice());
+  auto input_device = pm->model_nodes.get_node_by_name(DbStreamInputs::inputDevice());
 
   if (input_device.serial == SPA_ID_INVALID) {
     util::debug(std::format("The input device {} is not available. Aborting the link...",
-                            db::StreamInputs::inputDevice().toStdString()));
+                            DbStreamInputs::inputDevice().toStdString()));
 
     return;
   }
 
-  const auto list = (bypass) ? QStringList() : db::StreamInputs::plugins();
+  const auto list = (bypass) ? QStringList() : DbStreamInputs::plugins();
 
   auto mic_linked = false;
 
@@ -242,7 +220,7 @@ void StreamInputEffects::connect_filters(const bool& bypass) {
 
   int timeout = 0;
 
-  util::warning(std::format("Before: {} -> {}", input_device.id, input_device.name.toStdString()));
+  util::debug(std::format("Before: {} -> {}", input_device.id, input_device.name.toStdString()));
 
   while (pm->count_node_ports(input_device.id) < 1) {
     std::this_thread::sleep_for(std::chrono::milliseconds(1));
@@ -299,7 +277,7 @@ void StreamInputEffects::connect_filters(const bool& bypass) {
 
       if (name.startsWith(tags::plugin_name::BaseName::echoCanceller)) {
         if (plugins[name]->connected_to_pw) {
-          auto output_device = pm->model_nodes.get_node_by_name(db::StreamOutputs::outputDevice());
+          auto output_device = pm->model_nodes.get_node_by_name(DbStreamOutputs::outputDevice());
 
           for (const auto& link : pm->link_nodes(output_device.id, plugins[name]->get_node_id(), true)) {
             list_proxies.push_back(link);
@@ -331,12 +309,21 @@ void StreamInputEffects::connect_filters(const bool& bypass) {
       util::warning(std::format("Link from node {} to node {} failed", prev_node_id, next_node_id));
     }
   }
+
+  /*
+   * The correct thing to do would be to check if at least one pair of filters are actually linked. But filtersLinked
+   * is used only to enable or disable qml frame animation. So let's keep the approach simple.
+   */
+
+  filtersLinked = true;
+
+  Q_EMIT filtersLinkedChanged();
 }
 
 void StreamInputEffects::disconnect_filters() {
   std::set<uint> link_id_list;
 
-  const auto selected_plugins_list = (bypass) ? QStringList() : db::StreamInputs::plugins();
+  const auto selected_plugins_list = (bypass) ? QStringList() : DbStreamInputs::plugins();
 
   for (const auto& plugin : plugins | std::views::values) {
     if (plugin == nullptr) {
@@ -356,6 +343,8 @@ void StreamInputEffects::disconnect_filters() {
         plugin->disconnect_from_pw();
       }
     }
+
+    plugin->clear_data();
   }
 
   for (const auto& link : pm->get_links()) {
@@ -374,6 +363,10 @@ void StreamInputEffects::disconnect_filters() {
   list_proxies.clear();
 
   remove_unused_filters();
+
+  filtersLinked = false;
+
+  Q_EMIT filtersLinkedChanged();
 }
 
 void StreamInputEffects::set_bypass(const bool& state) {
@@ -386,8 +379,8 @@ void StreamInputEffects::set_bypass(const bool& state) {
 
 void StreamInputEffects::set_listen_to_mic(const bool& state) {
   if (state) {
-    auto output_device = !db::StreamInputs::listenToMicIncludesOutputEffects()
-                             ? pm->model_nodes.get_node_by_name(db::StreamOutputs::outputDevice())
+    auto output_device = !DbStreamInputs::listenToMicIncludesOutputEffects()
+                             ? pm->model_nodes.get_node_by_name(DbStreamOutputs::outputDevice())
                              : pm->ee_sink_node;
 
     for (const auto& link : pm->link_nodes(pm->ee_source_node.id, output_device.id, false, false)) {
